"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SiteClient = exports.ParsedSite = void 0;
const vscode_azureextensionui_1 = require("vscode-azureextensionui");
const AppKind_1 = require("./createAppService/AppKind");
const slotFunctionOperations_1 = require("./slotFunctionOperations");
const tryGetSiteResource_1 = require("./tryGetSiteResource");
const azureClients_1 = require("./utils/azureClients");
const nonNull_1 = require("./utils/nonNull");
class ParsedSite {
    constructor(site, subscription) {
        this.rawSite = site;
        let matches = nonNull_1.nonNullProp(site, 'serverFarmId').match(/\/subscriptions\/(.*)\/resourceGroups\/(.*)\/providers\/Microsoft.Web\/serverfarms\/(.*)/);
        matches = nonNull_1.nonNullValue(matches, 'Invalid serverFarmId.');
        this.id = nonNull_1.nonNullProp(site, 'id');
        [this.siteName, this.slotName] = nonNull_1.nonNullProp(site, 'name').split('/');
        this.isSlot = !!this.slotName;
        this.fullName = this.siteName + (this.slotName ? `-${this.slotName}` : '');
        this.resourceGroup = nonNull_1.nonNullProp(site, 'resourceGroup');
        this.location = site.location;
        this.serverFarmId = nonNull_1.nonNullProp(site, 'serverFarmId');
        this.kind = nonNull_1.nonNullProp(site, 'kind');
        this.initialState = site.state;
        const kind = (site.kind || '').toLowerCase();
        this.isFunctionApp = kind.includes(AppKind_1.AppKind.functionapp);
        this.isWorkflowApp = kind.includes(AppKind_1.AppKind.workflowapp);
        this.isKubernetesApp = kind.includes('kubernetes');
        this.isLinux = kind.includes('linux');
        this.planResourceGroup = matches[2];
        this.planName = matches[3];
        this.defaultHostName = nonNull_1.nonNullProp(site, 'defaultHostName');
        this.defaultHostUrl = `https://${this.defaultHostName}`;
        const kuduRepositoryUrl = nonNull_1.nonNullProp(site, 'hostNameSslStates').find(h => !!h.hostType && h.hostType.toLowerCase() === 'repository');
        if (kuduRepositoryUrl) {
            this.kuduHostName = kuduRepositoryUrl.name;
            this.kuduUrl = `https://${this.kuduHostName}`;
            this.gitUrl = `${this.kuduHostName}:443/${site.repositorySiteName}.git`;
        }
        this.subscription = subscription;
    }
    createClient(context) {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            let client = (_a = context._parsedSiteClients) === null || _a === void 0 ? void 0 : _a[this.id];
            if (!client) {
                const internalClient = yield azureClients_1.createWebSiteClient([context, this.subscription]);
                const internalGenericClient = yield vscode_azureextensionui_1.createGenericClient(context, this.subscription);
                client = new SiteClient(internalClient, internalGenericClient, this);
                context._parsedSiteClients || (context._parsedSiteClients = {});
                context._parsedSiteClients[this.id] = client;
            }
            return client;
        });
    }
}
exports.ParsedSite = ParsedSite;
/**
 * Wrapper of a WebSiteManagementClient for use with a specific Site
 * Reduces the number of arguments needed for every call and automatically ensures the 'slot' method is called when appropriate
 */
class SiteClient {
    constructor(internalClient, internalGenericClient, site) {
        this._client = internalClient;
        this._genericClient = internalGenericClient;
        this._site = site;
    }
    get fullName() {
        return this._site.fullName;
    }
    get isLinux() {
        return this._site.isLinux;
    }
    getIsConsumption(context) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this._site.isFunctionApp) {
                const sku = yield this.getCachedSku(context);
                return !!sku && sku.toLowerCase() === 'dynamic';
            }
            else {
                return false;
            }
        });
    }
    stop() {
        return __awaiter(this, void 0, void 0, function* () {
            this._site.slotName ?
                yield this._client.webApps.stopSlot(this._site.resourceGroup, this._site.siteName, this._site.slotName) :
                yield this._client.webApps.stop(this._site.resourceGroup, this._site.siteName);
        });
    }
    start() {
        return __awaiter(this, void 0, void 0, function* () {
            this._site.slotName ?
                yield this._client.webApps.startSlot(this._site.resourceGroup, this._site.siteName, this._site.slotName) :
                yield this._client.webApps.start(this._site.resourceGroup, this._site.siteName);
        });
    }
    getSite() {
        return __awaiter(this, void 0, void 0, function* () {
            return (this._site.slotName ?
                yield tryGetSiteResource_1.tryGetWebAppSlot(this._client, this._site.resourceGroup, this._site.siteName, this._site.slotName) :
                yield tryGetSiteResource_1.tryGetWebApp(this._client, this._site.resourceGroup, this._site.siteName));
        });
    }
    getState() {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            return (_a = (yield this.getSite())) === null || _a === void 0 ? void 0 : _a.state;
        });
    }
    getWebAppPublishCredential() {
        return __awaiter(this, void 0, void 0, function* () {
            return this._site.slotName ?
                yield this._client.webApps.listPublishingCredentialsSlot(this._site.resourceGroup, this._site.siteName, this._site.slotName) :
                yield this._client.webApps.listPublishingCredentials(this._site.resourceGroup, this._site.siteName);
        });
    }
    getSiteConfig() {
        return __awaiter(this, void 0, void 0, function* () {
            return this._site.slotName ?
                yield this._client.webApps.getConfigurationSlot(this._site.resourceGroup, this._site.siteName, this._site.slotName) :
                yield this._client.webApps.getConfiguration(this._site.resourceGroup, this._site.siteName);
        });
    }
    updateConfiguration(config) {
        return __awaiter(this, void 0, void 0, function* () {
            return this._site.slotName ?
                yield this._client.webApps.updateConfigurationSlot(this._site.resourceGroup, this._site.siteName, config, this._site.slotName) :
                yield this._client.webApps.updateConfiguration(this._site.resourceGroup, this._site.siteName, config);
        });
    }
    getLogsConfig() {
        return __awaiter(this, void 0, void 0, function* () {
            return this._site.slotName ?
                yield this._client.webApps.getDiagnosticLogsConfigurationSlot(this._site.resourceGroup, this._site.siteName, this._site.slotName) :
                yield this._client.webApps.getDiagnosticLogsConfiguration(this._site.resourceGroup, this._site.siteName);
        });
    }
    updateLogsConfig(config) {
        return __awaiter(this, void 0, void 0, function* () {
            return this._site.slotName ?
                yield this._client.webApps.updateDiagnosticLogsConfigSlot(this._site.resourceGroup, this._site.siteName, config, this._site.slotName) :
                yield this._client.webApps.updateDiagnosticLogsConfig(this._site.resourceGroup, this._site.siteName, config);
        });
    }
    getAppServicePlan() {
        return __awaiter(this, void 0, void 0, function* () {
            return yield tryGetSiteResource_1.tryGetAppServicePlan(this._client, this._site.planResourceGroup, this._site.planName);
        });
    }
    getSourceControl() {
        return __awaiter(this, void 0, void 0, function* () {
            return this._site.slotName ?
                yield this._client.webApps.getSourceControlSlot(this._site.resourceGroup, this._site.siteName, this._site.slotName) :
                yield this._client.webApps.getSourceControl(this._site.resourceGroup, this._site.siteName);
        });
    }
    updateSourceControl(siteSourceControl) {
        return __awaiter(this, void 0, void 0, function* () {
            return this._site.slotName ?
                yield this._client.webApps.createOrUpdateSourceControlSlot(this._site.resourceGroup, this._site.siteName, siteSourceControl, this._site.slotName) :
                yield this._client.webApps.createOrUpdateSourceControl(this._site.resourceGroup, this._site.siteName, siteSourceControl);
        });
    }
    syncRepository() {
        return __awaiter(this, void 0, void 0, function* () {
            this._site.slotName ?
                yield this._client.webApps.syncRepositorySlot(this._site.resourceGroup, this._site.siteName, this._site.slotName) :
                yield this._client.webApps.syncRepository(this._site.resourceGroup, this._site.siteName);
        });
    }
    listApplicationSettings() {
        return __awaiter(this, void 0, void 0, function* () {
            return this._site.slotName ?
                yield this._client.webApps.listApplicationSettingsSlot(this._site.resourceGroup, this._site.siteName, this._site.slotName) :
                yield this._client.webApps.listApplicationSettings(this._site.resourceGroup, this._site.siteName);
        });
    }
    updateApplicationSettings(appSettings) {
        return __awaiter(this, void 0, void 0, function* () {
            return this._site.slotName ?
                yield this._client.webApps.updateApplicationSettingsSlot(this._site.resourceGroup, this._site.siteName, appSettings, this._site.slotName) :
                yield this._client.webApps.updateApplicationSettings(this._site.resourceGroup, this._site.siteName, appSettings);
        });
    }
    listSlotConfigurationNames() {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this._client.webApps.listSlotConfigurationNames(this._site.resourceGroup, this._site.siteName);
        });
    }
    updateSlotConfigurationNames(appSettings) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this._client.webApps.updateSlotConfigurationNames(this._site.resourceGroup, this._site.siteName, appSettings);
        });
    }
    deleteMethod(options) {
        return __awaiter(this, void 0, void 0, function* () {
            this._site.slotName ?
                yield this._client.webApps.deleteSlot(this._site.resourceGroup, this._site.siteName, this._site.slotName, options) :
                yield this._client.webApps.deleteMethod(this._site.resourceGroup, this._site.siteName, options);
        });
    }
    listInstanceIdentifiers() {
        return __awaiter(this, void 0, void 0, function* () {
            return this._site.slotName ?
                yield this._client.webApps.listInstanceIdentifiersSlot(this._site.resourceGroup, this._site.siteName, this._site.slotName) :
                yield this._client.webApps.listInstanceIdentifiers(this._site.resourceGroup, this._site.siteName);
        });
    }
    listSourceControls() {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this._client.listSourceControls();
        });
    }
    listFunctions() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this._site.slotName) {
                return yield slotFunctionOperations_1.listFunctionsSlot(this._genericClient, this._site.id);
            }
            else {
                return yield this._client.webApps.listFunctions(this._site.resourceGroup, this._site.siteName);
            }
        });
    }
    listFunctionsNext(nextPageLink) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this._client.webApps.listFunctionsNext(nextPageLink);
        });
    }
    getFunction(functionName) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this._site.slotName) {
                return yield slotFunctionOperations_1.getFunctionSlot(this._genericClient, this._site.id, functionName);
            }
            else {
                return yield this._client.webApps.getFunction(this._site.resourceGroup, this._site.siteName, functionName);
            }
        });
    }
    deleteFunction(functionName) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this._site.slotName) {
                yield slotFunctionOperations_1.deleteFunctionSlot(this._genericClient, this._site.id, functionName);
            }
            else {
                yield this._client.webApps.deleteFunction(this._site.resourceGroup, this._site.siteName, functionName);
            }
        });
    }
    listFunctionSecrets(functionName) {
        return __awaiter(this, void 0, void 0, function* () {
            return this._site.slotName ?
                yield this._client.webApps.listFunctionSecretsSlot(this._site.resourceGroup, this._site.siteName, functionName, this._site.slotName) :
                yield this._client.webApps.listFunctionSecrets(this._site.resourceGroup, this._site.siteName, functionName);
        });
    }
    syncFunctionTriggers() {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                this._site.slotName ?
                    yield this._client.webApps.syncFunctionTriggersSlot(this._site.resourceGroup, this._site.siteName, this._site.slotName) :
                    yield this._client.webApps.syncFunctionTriggers(this._site.resourceGroup, this._site.siteName);
            }
            catch (error) {
                // For some reason this call incorrectly throws an error when the status code is 200
                if (vscode_azureextensionui_1.parseError(error).errorType !== '200') {
                    throw error;
                }
            }
        });
    }
    getPublishingUser() {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this._client.getPublishingUser({});
        });
    }
    listWebJobs() {
        return __awaiter(this, void 0, void 0, function* () {
            return this._site.slotName ?
                yield this._client.webApps.listWebJobsSlot(this._site.resourceGroup, this._site.siteName, this._site.slotName) :
                yield this._client.webApps.listWebJobs(this._site.resourceGroup, this._site.siteName);
        });
    }
    listHostKeys() {
        return __awaiter(this, void 0, void 0, function* () {
            return this._site.slotName ?
                yield this._client.webApps.listHostKeysSlot(this._site.resourceGroup, this._site.siteName, this._site.slotName) :
                yield this._client.webApps.listHostKeys(this._site.resourceGroup, this._site.siteName);
        });
    }
    listFunctionKeys(functionName) {
        return __awaiter(this, void 0, void 0, function* () {
            return this._site.slotName ?
                yield this._client.webApps.listFunctionKeysSlot(this._site.resourceGroup, this._site.siteName, functionName, this._site.slotName) :
                yield this._client.webApps.listFunctionKeys(this._site.resourceGroup, this._site.siteName, functionName);
        });
    }
    /**
     * Temporary workaround because the azure sdk doesn't return the full site object from Azure
     * Hopefully this can be removed when we move to the new sdk
     * Also, we're caching the sku - for better performance and because it's unlikely to change
     */
    getCachedSku(context) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this._cachedSku) {
                const client = yield vscode_azureextensionui_1.createGenericClient(context, this._site.subscription);
                const response = yield client.sendRequest({ method: 'GET', url: `${this._site.id}?api-version=2016-08-01` });
                this._cachedSku = response.parsedBody.properties.sku;
            }
            return this._cachedSku;
        });
    }
}
exports.SiteClient = SiteClient;
//# sourceMappingURL=SiteClient.js.map