"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.TunnelProxy = void 0;
const ms_rest_js_1 = require("@azure/ms-rest-js");
const EventEmitter = require("events");
const net_1 = require("net");
const vscode_azureextensionui_1 = require("vscode-azureextensionui");
const websocket = require("websocket");
const extensionVariables_1 = require("./extensionVariables");
const localize_1 = require("./localize");
const delay_1 = require("./utils/delay");
const nonNull_1 = require("./utils/nonNull");
/**
 * Wrapper for net.Socket that forwards all traffic to the Kudu tunnel websocket endpoint.
 * Used internally by the TunnelProxy server.
 */
class TunnelSocket extends EventEmitter {
    constructor(socket, site, publishCredential) {
        super();
        this._socket = socket;
        this._site = site;
        this._publishCredential = publishCredential;
        this._wsClient = new websocket.client();
    }
    connect() {
        extensionVariables_1.ext.outputChannel.appendLog('[Proxy Server] socket init');
        // Pause socket until tunnel connection has been established to make sure we don't lose data
        this._socket.pause();
        this._socket.on('data', (data) => {
            if (this._wsConnection) {
                this._wsConnection.send(data);
            }
        });
        this._socket.on('close', () => {
            extensionVariables_1.ext.outputChannel.appendLog(`[Proxy Server] client disconnected ${this._socket.remoteAddress}:${this._socket.remotePort}`);
            this.dispose();
            this.emit('close');
        });
        this._socket.on('error', (err) => {
            extensionVariables_1.ext.outputChannel.appendLog(`[Proxy Server] socket error: ${err}`);
            this.dispose();
            this.emit('error', err);
        });
        this._wsClient.on('connect', (connection) => {
            extensionVariables_1.ext.outputChannel.appendLog('[WebSocket] client connected');
            this._wsConnection = connection;
            // Resume socket after connection
            this._socket.resume();
            connection.on('close', () => {
                extensionVariables_1.ext.outputChannel.appendLog('[WebSocket] client closed');
                this.dispose();
                this.emit('close');
            });
            connection.on('error', (err) => {
                extensionVariables_1.ext.outputChannel.appendLog(`[WebSocket] error: ${err}`);
                this.dispose();
                this.emit('error', err);
            });
            connection.on('message', (data) => {
                if (data.binaryData) {
                    this._socket.write(data.binaryData);
                }
            });
        });
        this._wsClient.on('connectFailed', (err) => {
            extensionVariables_1.ext.outputChannel.appendLog(`[WebSocket] connectFailed: ${err}`);
            this.dispose();
            this.emit('error', err);
        });
        this._wsClient.connect(`wss://${this._site.kuduHostName}/AppServiceTunnel/Tunnel.ashx`, undefined, undefined, {
            'User-Agent': 'vscode-azuretools',
            'Cache-Control': 'no-cache',
            Pragma: 'no-cache'
        }, {
            auth: `${this._publishCredential.publishingUserName}:${this._publishCredential.publishingPassword}`
        });
    }
    dispose() {
        extensionVariables_1.ext.outputChannel.appendLog('[Proxy Server] socket dispose');
        if (this._wsConnection) {
            this._wsConnection.close();
            this._wsConnection = undefined;
        }
        this._wsClient.abort();
        this._socket.destroy();
    }
}
/**
 * Interface for tunnel GetStatus API
 */
var AppState;
(function (AppState) {
    AppState["STARTED"] = "STARTED";
    AppState["STARTING"] = "STARTING";
    AppState["STOPPED"] = "STOPPED";
})(AppState || (AppState = {}));
/**
 * Internal error indicating that we should continue to retry getting the tunnel status
 */
class RetryableTunnelStatusError extends Error {
}
/**
 * A local TCP server that forwards all connections to the Kudu tunnel websocket endpoint.
 */
class TunnelProxy {
    constructor(port, site, publishCredential, isSsh = false) {
        this._port = port;
        this._site = site;
        this._publishCredential = publishCredential;
        this._server = net_1.createServer();
        this._openSockets = [];
        this._isSsh = isSsh;
    }
    startProxy(context, token) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                yield this.checkTunnelStatusWithRetry(context, token);
                yield this.setupTunnelServer(token);
            }
            catch (error) {
                this.dispose();
                throw error;
            }
        });
    }
    dispose() {
        this._openSockets.forEach((tunnelSocket) => {
            tunnelSocket.dispose();
        });
        this._server.close();
        this._server.unref();
    }
    /**
     * Ensures the site is up and running (Even if the state is technically "Running", Azure doesn't always keep a site fully "up" if no one has hit the url in a while)
     */
    pingApp(context) {
        return __awaiter(this, void 0, void 0, function* () {
            extensionVariables_1.ext.outputChannel.appendLog('[Tunnel] Pinging app default url...');
            const client = yield vscode_azureextensionui_1.createGenericClient(context, undefined);
            let statusCode;
            try {
                const response = yield client.sendRequest({ method: 'GET', url: this._site.defaultHostUrl });
                statusCode = response.status;
            }
            catch (error) {
                if (error instanceof ms_rest_js_1.RestError) {
                    statusCode = error.statusCode;
                }
                else {
                    throw error;
                }
            }
            extensionVariables_1.ext.outputChannel.appendLog(`[Tunnel] Ping responded with status code: ${statusCode}`);
        });
    }
    checkTunnelStatus(context) {
        return __awaiter(this, void 0, void 0, function* () {
            const password = nonNull_1.nonNullProp(this._publishCredential, 'publishingPassword');
            const client = yield vscode_azureextensionui_1.createGenericClient(context, new ms_rest_js_1.BasicAuthenticationCredentials(this._publishCredential.publishingUserName, password));
            let tunnelStatus;
            try {
                const response = yield client.sendRequest({
                    method: 'GET',
                    url: `https://${this._site.kuduHostName}/AppServiceTunnel/Tunnel.ashx?GetStatus&GetStatusAPIVer=2`
                });
                extensionVariables_1.ext.outputChannel.appendLog(`[Tunnel] Checking status, body: ${response.bodyAsText}`);
                tunnelStatus = response.parsedBody;
            }
            catch (error) {
                const parsedError = vscode_azureextensionui_1.parseError(error);
                extensionVariables_1.ext.outputChannel.appendLog(`[Tunnel] Checking status, error: ${parsedError.message}`);
                throw new Error(localize_1.localize('tunnelStatusError', 'Error getting tunnel status: {0}', parsedError.errorType));
            }
            if (tunnelStatus.state === AppState.STARTED) {
                if ((tunnelStatus.port === 2222 && !this._isSsh) || (tunnelStatus.port !== 2222 && this._isSsh)) {
                    // Tunnel is pointed to default SSH port and still needs time to restart
                    throw new RetryableTunnelStatusError();
                }
                else if (tunnelStatus.canReachPort) {
                    return;
                }
                else {
                    throw new Error(localize_1.localize('tunnelUnreachable', 'App is started, but port is unreachable'));
                }
            }
            else if (tunnelStatus.state === AppState.STARTING) {
                throw new RetryableTunnelStatusError();
            }
            else if (tunnelStatus.state === AppState.STOPPED) {
                yield this.pingApp(context);
                throw new RetryableTunnelStatusError();
            }
            else {
                throw new Error(localize_1.localize('tunnelStatusError', 'Unexpected app state: {0}', tunnelStatus.state));
            }
        });
    }
    checkTunnelStatusWithRetry(context, token) {
        return __awaiter(this, void 0, void 0, function* () {
            const timeoutSeconds = 240; // 4 minutes, matches App Service internal timeout for starting up an app
            const timeoutMs = timeoutSeconds * 1000;
            const pollingIntervalMs = 5000;
            const start = Date.now();
            while (Date.now() < start + timeoutMs) {
                if (token.isCancellationRequested) {
                    throw new vscode_azureextensionui_1.UserCancelledError('checkTunnelStatus');
                }
                yield this.pingApp(context);
                try {
                    yield this.checkTunnelStatus(context);
                    return;
                }
                catch (error) {
                    if (!(error instanceof RetryableTunnelStatusError)) {
                        throw new Error(localize_1.localize('tunnelFailed', 'Unable to establish connection to application: {0}', vscode_azureextensionui_1.parseError(error).message));
                    } // else allow retry
                }
                yield delay_1.delay(pollingIntervalMs);
            }
            throw new Error(localize_1.localize('tunnelTimedOut', 'Unable to establish connection to application: Timed out'));
        });
    }
    setupTunnelServer(token) {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => {
                const listener = token.onCancellationRequested(() => {
                    reject(new vscode_azureextensionui_1.UserCancelledError('setupTunnelServer'));
                    listener.dispose();
                });
                this._server.on('connection', (socket) => {
                    const tunnelSocket = new TunnelSocket(socket, this._site, this._publishCredential);
                    this._openSockets.push(tunnelSocket);
                    tunnelSocket.on('close', () => {
                        const index = this._openSockets.indexOf(tunnelSocket);
                        if (index >= 0) {
                            this._openSockets.splice(index, 1);
                            extensionVariables_1.ext.outputChannel.appendLog(`[Proxy Server] client closed, connection count: ${this._openSockets.length}`);
                        }
                    });
                    tunnelSocket.connect();
                    extensionVariables_1.ext.outputChannel.appendLog(`[Proxy Server] client connected ${socket.remoteAddress}:${socket.remotePort}, connection count: ${this._openSockets.length}`);
                });
                this._server.on('listening', () => {
                    extensionVariables_1.ext.outputChannel.appendLog('[Proxy Server] start listening');
                    resolve();
                });
                this._server.on('error', (err) => {
                    extensionVariables_1.ext.outputChannel.appendLog(`[Proxy Server] server error: ${err}`);
                    this.dispose();
                    reject(err);
                });
                this._server.listen({
                    host: 'localhost',
                    port: this._port,
                    backlog: 1
                });
            });
        });
    }
}
exports.TunnelProxy = TunnelProxy;
//# sourceMappingURL=TunnelProxy.js.map