"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.AppInsightsCreateStep = void 0;
const vscode_azureextensionui_1 = require("vscode-azureextensionui");
const extensionVariables_1 = require("../extensionVariables");
const localize_1 = require("../localize");
const azureClients_1 = require("../utils/azureClients");
const azureUtils_1 = require("../utils/azureUtils");
const nonNull_1 = require("../utils/nonNull");
const AppInsightsListStep_1 = require("./AppInsightsListStep");
class AppInsightsCreateStep extends vscode_azureextensionui_1.AzureWizardExecuteStep {
    constructor() {
        super(...arguments);
        this.priority = 135;
    }
    execute(context, progress) {
        return __awaiter(this, void 0, void 0, function* () {
            const resourceLocation = yield vscode_azureextensionui_1.LocationListStep.getLocation(context);
            const verifyingAppInsightsAvailable = localize_1.localize('verifyingAppInsightsAvailable', 'Verifying that Application Insights is available for this location...');
            extensionVariables_1.ext.outputChannel.appendLog(verifyingAppInsightsAvailable);
            const appInsightsLocation = yield this.getSupportedLocation(context, resourceLocation);
            if (appInsightsLocation) {
                const client = yield azureClients_1.createAppInsightsClient(context);
                const rg = nonNull_1.nonNullProp(context, 'resourceGroup');
                const rgName = nonNull_1.nonNullProp(rg, 'name');
                const aiName = nonNull_1.nonNullProp(context, 'newAppInsightsName');
                try {
                    context.appInsightsComponent = yield client.components.get(rgName, aiName);
                    extensionVariables_1.ext.outputChannel.appendLog(localize_1.localize('existingNewAppInsights', 'Using existing Application Insights resource "{0}".', aiName));
                }
                catch (error) {
                    const pError = vscode_azureextensionui_1.parseError(error);
                    // Only expecting a resource not found error if this is a new component
                    if (pError.errorType === 'ResourceNotFound') {
                        const creatingNewAppInsights = localize_1.localize('creatingNewAppInsightsInsights', 'Creating Application Insights resource "{0}"...', aiName);
                        extensionVariables_1.ext.outputChannel.appendLog(creatingNewAppInsights);
                        progress.report({ message: creatingNewAppInsights });
                        context.appInsightsComponent = yield client.components.createOrUpdate(rgName, aiName, { kind: 'web', applicationType: 'web', location: appInsightsLocation });
                        const createdNewAppInsights = localize_1.localize('createdNewAppInsights', 'Successfully created Application Insights resource "{0}".', aiName);
                        extensionVariables_1.ext.outputChannel.appendLog(createdNewAppInsights);
                    }
                    else if (pError.errorType === 'AuthorizationFailed') {
                        if (!context.advancedCreation) {
                            const appInsightsNotAuthorized = localize_1.localize('appInsightsNotAuthorized', 'Skipping Application Insights resource because you do not have permission to create one in this subscription.');
                            extensionVariables_1.ext.outputChannel.appendLog(appInsightsNotAuthorized);
                        }
                        else {
                            yield this.selectExistingPrompt(context);
                        }
                    }
                    else {
                        throw error;
                    }
                }
            }
            else {
                const appInsightsNotAvailable = localize_1.localize('appInsightsNotAvailable', 'Skipping Application Insights resource because it isn\'t compatible with this location.');
                extensionVariables_1.ext.outputChannel.appendLog(appInsightsNotAvailable);
            }
        });
    }
    selectExistingPrompt(context) {
        return __awaiter(this, void 0, void 0, function* () {
            const message = localize_1.localize('aiForbidden', 'You do not have permission to create an app insights resource in subscription "{0}".', context.subscriptionDisplayName);
            const selectExisting = { title: localize_1.localize('selectExisting', 'Select Existing') };
            const skipForNow = { title: localize_1.localize('skipForNow', 'Skip for Now') };
            const result = yield context.ui.showWarningMessage(message, { modal: true, stepName: 'AppInsightsNoPermissions' }, selectExisting, skipForNow);
            if (result === skipForNow) {
                context.telemetry.properties.aiSkipForNow = 'true';
                context.appInsightsSkip = true;
                context.telemetry.properties.forbiddenResponse = 'SkipAppInsights';
            }
            else {
                context.telemetry.properties.forbiddenResponse = 'SelectExistingAppInsights';
                const step = new AppInsightsListStep_1.AppInsightsListStep(true /* suppressCreate */);
                yield step.prompt(context);
            }
        });
    }
    shouldExecute(context) {
        return !context.appInsightsComponent && !!context.newAppInsightsName;
    }
    // returns the supported location, a location in the region map, or undefined
    getSupportedLocation(context, location) {
        return __awaiter(this, void 0, void 0, function* () {
            const locations = (yield this.getLocations(context)) || [];
            const locationName = nonNull_1.nonNullProp(location, 'name');
            if (locations.some((loc) => azureUtils_1.areLocationNamesEqual(loc, location.name))) {
                context.telemetry.properties.aiLocationSupported = 'true';
                return locationName;
            }
            else {
                // If there is no exact match, then query the regionMapping.json
                const pairedRegions = yield this.getPairedRegions(context, locationName);
                if (pairedRegions.length > 0) {
                    // if there is at least one region listed, return the first
                    context.telemetry.properties.aiLocationSupported = 'pairedRegion';
                    return pairedRegions[0];
                }
                context.telemetry.properties.aiLocationSupported = 'false';
                return undefined;
            }
        });
    }
    getPairedRegions(context, locationName) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const client = yield vscode_azureextensionui_1.createGenericClient(context, undefined);
                const response = yield client.sendRequest({
                    method: 'GET',
                    url: 'https://appinsights.azureedge.net/portal/regionMapping.json'
                });
                const regionMappingJson = response.parsedBody;
                if (regionMappingJson.regions[locationName]) {
                    return regionMappingJson.regions[locationName].pairedRegions;
                }
            }
            catch (error) {
                // ignore the error
            }
            return [];
        });
    }
    getLocations(context) {
        return __awaiter(this, void 0, void 0, function* () {
            const resourceClient = yield azureClients_1.createResourceClient(context);
            const supportedRegions = yield resourceClient.providers.get('microsoft.insights');
            const componentsResourceType = supportedRegions.resourceTypes && supportedRegions.resourceTypes.find(aiRt => aiRt.resourceType === 'components');
            if (!!componentsResourceType && !!componentsResourceType.locations) {
                return componentsResourceType.locations;
            }
            else {
                return undefined;
            }
        });
    }
}
exports.AppInsightsCreateStep = AppInsightsCreateStep;
//# sourceMappingURL=AppInsightsCreateStep.js.map