"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.AppInsightsListStep = exports.appInsightsNamingRules = void 0;
const vscode_azureextensionui_1 = require("vscode-azureextensionui");
const localize_1 = require("../localize");
const azureClients_1 = require("../utils/azureClients");
const nonNull_1 = require("../utils/nonNull");
const AppInsightsCreateStep_1 = require("./AppInsightsCreateStep");
const AppInsightsNameStep_1 = require("./AppInsightsNameStep");
exports.appInsightsNamingRules = {
    minLength: 1,
    maxLength: 255,
    invalidCharsRegExp: /[^a-zA-Z0-9\.\_\-\(\)]/
};
const skipForNowLabel = '$(clock) Skip for now';
class AppInsightsListStep extends vscode_azureextensionui_1.AzureWizardPromptStep {
    constructor(suppressCreate) {
        super();
        this._suppressCreate = suppressCreate;
    }
    static getAppInsightsComponents(context) {
        return __awaiter(this, void 0, void 0, function* () {
            if (context.appInsightsTask === undefined) {
                const client = yield azureClients_1.createAppInsightsClient(context);
                context.appInsightsTask = client.components.list();
            }
            return yield context.appInsightsTask;
        });
    }
    prompt(context) {
        return __awaiter(this, void 0, void 0, function* () {
            const options = { placeHolder: 'Select an Application Insights resource for your app.', id: `AppInsightsListStep/${context.subscriptionId}` };
            const input = (yield context.ui.showQuickPick(this.getQuickPicks(context), options));
            context.appInsightsComponent = input.data;
            // as create new and skipForNow both have undefined as the data type, check the label
            if (input.label === skipForNowLabel) {
                context.telemetry.properties.aiSkipForNow = 'true';
                context.appInsightsSkip = true;
            }
            else {
                context.telemetry.properties.newAI = String(!context.appInsightsComponent);
            }
        });
    }
    shouldPrompt(context) {
        return !context.appInsightsComponent;
    }
    getSubWizard(context) {
        return __awaiter(this, void 0, void 0, function* () {
            if (context.appInsightsComponent) {
                context.valuesToMask.push(nonNull_1.nonNullProp(context.appInsightsComponent, 'name'));
            }
            else if (!context.appInsightsSkip) {
                const promptSteps = [new AppInsightsNameStep_1.AppInsightsNameStep()];
                vscode_azureextensionui_1.LocationListStep.addStep(context, promptSteps);
                return {
                    promptSteps: promptSteps,
                    executeSteps: [new AppInsightsCreateStep_1.AppInsightsCreateStep()]
                };
            }
            return undefined;
        });
    }
    getQuickPicks(context) {
        return __awaiter(this, void 0, void 0, function* () {
            const picks = !this._suppressCreate ? [{
                    label: localize_1.localize('newApplicationInsight', '$(plus) Create new Application Insights resource'),
                    data: undefined
                }] : [];
            picks.push({
                label: localize_1.localize('skipForNow', skipForNowLabel),
                data: undefined
            });
            let components = yield AppInsightsListStep.getAppInsightsComponents(context);
            // https://github.com/microsoft/vscode-azurefunctions/issues/1454
            if (!Array.isArray(components)) {
                components = [];
            }
            return picks.concat(components.map((ai) => {
                return {
                    id: ai.id,
                    label: nonNull_1.nonNullProp(ai, 'name'),
                    description: ai.location,
                    data: ai
                };
            }));
        });
    }
}
exports.AppInsightsListStep = AppInsightsListStep;
//# sourceMappingURL=AppInsightsListStep.js.map