"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.AppServicePlanCreateStep = void 0;
const arm_appservice_1 = require("@azure/arm-appservice");
const vscode_azureextensionui_1 = require("vscode-azureextensionui");
const constants_1 = require("../constants");
const extensionVariables_1 = require("../extensionVariables");
const localize_1 = require("../localize");
const tryGetSiteResource_1 = require("../tryGetSiteResource");
const azureClients_1 = require("../utils/azureClients");
const nonNull_1 = require("../utils/nonNull");
const AppKind_1 = require("./AppKind");
const AppServicePlanListStep_1 = require("./AppServicePlanListStep");
class AppServicePlanCreateStep extends vscode_azureextensionui_1.AzureWizardExecuteStep {
    constructor() {
        super(...arguments);
        this.priority = 120;
    }
    execute(context, progress) {
        return __awaiter(this, void 0, void 0, function* () {
            const newPlanName = nonNull_1.nonNullProp(context, 'newPlanName');
            const rgName = nonNull_1.nonNullValueAndProp(context.resourceGroup, 'name');
            const findingAppServicePlan = localize_1.localize('FindingAppServicePlan', 'Ensuring App Service plan "{0}" exists...', newPlanName);
            const creatingAppServicePlan = localize_1.localize('CreatingAppServicePlan', 'Creating App Service plan "{0}"...', newPlanName);
            const foundAppServicePlan = localize_1.localize('FoundAppServicePlan', 'Successfully found App Service plan "{0}".', newPlanName);
            const createdAppServicePlan = localize_1.localize('CreatedAppServicePlan', 'Successfully created App Service plan "{0}".', newPlanName);
            extensionVariables_1.ext.outputChannel.appendLog(findingAppServicePlan);
            try {
                const client = yield azureClients_1.createWebSiteClient(context);
                const existingPlan = yield tryGetSiteResource_1.tryGetAppServicePlan(client, rgName, newPlanName);
                if (existingPlan) {
                    context.plan = existingPlan;
                    extensionVariables_1.ext.outputChannel.appendLog(foundAppServicePlan);
                }
                else {
                    extensionVariables_1.ext.outputChannel.appendLog(creatingAppServicePlan);
                    progress.report({ message: creatingAppServicePlan });
                    context.plan = yield client.appServicePlans.createOrUpdate(rgName, newPlanName, yield getNewPlan(context));
                    extensionVariables_1.ext.outputChannel.appendLog(createdAppServicePlan);
                }
            }
            catch (e) {
                if (vscode_azureextensionui_1.parseError(e).errorType === 'AuthorizationFailed') {
                    yield this.selectExistingPrompt(context);
                }
                else {
                    throw e;
                }
            }
        });
    }
    selectExistingPrompt(context) {
        return __awaiter(this, void 0, void 0, function* () {
            const message = localize_1.localize('planForbidden', 'You do not have permission to create an app service plan in subscription "{0}".', context.subscriptionDisplayName);
            const selectExisting = { title: localize_1.localize('selectExisting', 'Select Existing') };
            yield context.ui.showWarningMessage(message, { modal: true, stepName: 'AspNoPermissions' }, selectExisting);
            context.telemetry.properties.forbiddenResponse = 'SelectExistingAsp';
            const step = new AppServicePlanListStep_1.AppServicePlanListStep(true /* suppressCreate */);
            yield step.prompt(context);
        });
    }
    shouldExecute(context) {
        return !context.plan;
    }
}
exports.AppServicePlanCreateStep = AppServicePlanCreateStep;
function getNewPlan(context) {
    var _a, _b;
    return __awaiter(this, void 0, void 0, function* () {
        const location = yield vscode_azureextensionui_1.LocationListStep.getLocation(context, constants_1.webProvider);
        const plan = {
            kind: getPlanKind(context),
            sku: nonNull_1.nonNullProp(context, 'newPlanSku'),
            location: location.name,
            reserved: context.newSiteOS === AppKind_1.WebsiteOS.linux // The secret property - must be set to true to make it a Linux plan. Confirmed by the team who owns this API.
        };
        const skuFamily = ((_a = context.newPlanSku) === null || _a === void 0 ? void 0 : _a.family) ? (_b = context.newPlanSku) === null || _b === void 0 ? void 0 : _b.family.toLowerCase() : '';
        if (skuFamily === 'ep' || skuFamily === 'ws') {
            plan.maximumElasticWorkerCount = 20;
        }
        if (context.customLocation) {
            addCustomLocationProperties(plan, context.customLocation);
        }
        return plan;
    });
}
/**
 * Has a few temporary workarounds so that the sdk allows some newer properties on the plan
 */
function addCustomLocationProperties(plan, customLocation) {
    plan.perSiteScaling = true;
    // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
    arm_appservice_1.WebSiteManagementMappers.AppServicePlan.type.modelProperties.kubeEnvironmentProfile = {
        serializedName: 'properties.kubeEnvironmentProfile',
        type: {
            name: "Composite",
            modelProperties: {
                id: {
                    serializedName: "id",
                    type: {
                        name: "String"
                    }
                }
            }
        }
    };
    // eslint-disable-next-line @typescript-eslint/no-unsafe-member-access, @typescript-eslint/no-explicit-any
    plan.kubeEnvironmentProfile = { id: customLocation.kubeEnvironment.id };
    // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
    arm_appservice_1.WebSiteManagementMappers.AppServicePlan.type.modelProperties.extendedLocation = {
        serializedName: 'extendedLocation',
        type: {
            name: "Composite",
            modelProperties: {
                name: {
                    serializedName: "name",
                    type: {
                        name: "String"
                    }
                },
                type: {
                    serializedName: "type",
                    type: {
                        name: "String"
                    }
                }
            }
        }
    };
    // eslint-disable-next-line @typescript-eslint/no-unsafe-member-access, @typescript-eslint/no-explicit-any
    plan.extendedLocation = { name: customLocation.id, type: 'customLocation' };
}
function getPlanKind(context) {
    if (context.customLocation) {
        return 'linux,kubernetes';
    }
    else if (context.newSiteOS === AppKind_1.WebsiteOS.linux) {
        return AppKind_1.WebsiteOS.linux;
    }
    else {
        return AppKind_1.AppKind.app;
    }
}
//# sourceMappingURL=AppServicePlanCreateStep.js.map