"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.AppServicePlanListStep = void 0;
const vscode_azureextensionui_1 = require("vscode-azureextensionui");
const constants_1 = require("../constants");
const localize_1 = require("../localize");
const tryGetSiteResource_1 = require("../tryGetSiteResource");
const azureClients_1 = require("../utils/azureClients");
const nonNull_1 = require("../utils/nonNull");
const uiUtils_1 = require("../utils/uiUtils");
const AppKind_1 = require("./AppKind");
const AppServicePlanCreateStep_1 = require("./AppServicePlanCreateStep");
const AppServicePlanNameStep_1 = require("./AppServicePlanNameStep");
const AppServicePlanSkuStep_1 = require("./AppServicePlanSkuStep");
class AppServicePlanListStep extends vscode_azureextensionui_1.AzureWizardPromptStep {
    constructor(suppressCreate) {
        super();
        this._suppressCreate = suppressCreate;
    }
    static getPlans(context) {
        return __awaiter(this, void 0, void 0, function* () {
            if (context.plansTask === undefined) {
                const client = yield azureClients_1.createWebSiteClient(context);
                context.plansTask = uiUtils_1.uiUtils.listAll(client.appServicePlans, client.appServicePlans.list());
            }
            return yield context.plansTask;
        });
    }
    static isNameAvailable(context, name, resourceGroupName) {
        return __awaiter(this, void 0, void 0, function* () {
            const plans = yield AppServicePlanListStep.getPlans(context);
            return !plans.some(plan => nonNull_1.nonNullProp(plan, 'resourceGroup').toLowerCase() === resourceGroupName.toLowerCase() &&
                nonNull_1.nonNullProp(plan, 'name').toLowerCase() === name.toLowerCase());
        });
    }
    prompt(context) {
        var _a, _b;
        return __awaiter(this, void 0, void 0, function* () {
            // Cache hosting plan separately per subscription
            // Logic Apps only supports Workflow Standard sku and for App Service Plan it only supports one Isolated sku.
            // Since create is not enabled for isolated skus, we explicitly reference the type of plan picked in the placeHolder.
            const options = {
                placeHolder: ((_a = context.newSiteKind) === null || _a === void 0 ? void 0 : _a.includes(AppKind_1.AppKind.workflowapp)) && ((_b = context.planSkuFamilyFilter) === null || _b === void 0 ? void 0 : _b.test('IV2'))
                    ? localize_1.localize('selectV3Plan', 'Select an App Service Environment (v3) Plan')
                    : localize_1.localize('selectPlan', 'Select a {0} App Service plan.', AppKind_1.getWebsiteOSDisplayName(nonNull_1.nonNullProp(context, 'newSiteOS'))),
                id: `AppServicePlanListStep/${context.subscriptionId}`
            };
            context.plan = (yield context.ui.showQuickPick(this.getQuickPicks(context), options)).data;
            context.telemetry.properties.newPlan = String(!context.plan);
            if (context.plan) {
                yield vscode_azureextensionui_1.LocationListStep.setLocation(context, context.plan.location);
            }
        });
    }
    getSubWizard(context) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!context.plan) {
                const promptSteps = [new AppServicePlanNameStep_1.AppServicePlanNameStep(), new AppServicePlanSkuStep_1.AppServicePlanSkuStep(), new vscode_azureextensionui_1.ResourceGroupListStep()];
                vscode_azureextensionui_1.LocationListStep.addStep(context, promptSteps);
                return {
                    promptSteps: promptSteps,
                    executeSteps: [new AppServicePlanCreateStep_1.AppServicePlanCreateStep()]
                };
            }
            else {
                context.valuesToMask.push(nonNull_1.nonNullProp(context.plan, 'name'));
                return undefined;
            }
        });
    }
    shouldPrompt(context) {
        return !context.plan && !context.newPlanName;
    }
    getQuickPicks(context) {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            const picks = !this._suppressCreate
                ? [{
                        label: localize_1.localize('CreateNewAppServicePlan', '$(plus) Create new App Service plan'),
                        description: '',
                        data: undefined
                    }]
                : [];
            let plans = yield AppServicePlanListStep.getPlans(context);
            const famFilter = context.planSkuFamilyFilter;
            if (famFilter) {
                plans = plans.filter(plan => !plan.sku || !plan.sku.family || famFilter.test(plan.sku.family));
            }
            let location;
            if (vscode_azureextensionui_1.LocationListStep.hasLocation(context)) {
                location = yield vscode_azureextensionui_1.LocationListStep.getLocation(context, constants_1.webProvider);
            }
            let hasFilteredLocations = false;
            for (const plan of plans) {
                const isNewSiteLinux = context.newSiteOS === AppKind_1.WebsiteOS.linux;
                let isPlanLinux = nonNull_1.nonNullProp(plan, 'kind').toLowerCase().includes(AppKind_1.WebsiteOS.linux);
                if (plan.sku && (plan.sku.family === 'EP' || plan.sku.family === 'WS')) {
                    // elastic premium plans and workflow standard plans do not have the os in the kind, so we have to check the "reserved" property
                    // also, the "reserved" property is always "false" in the list of plans returned above. We have to perform a separate get on each plan
                    const client = yield azureClients_1.createWebSiteClient(context);
                    const epPlan = yield tryGetSiteResource_1.tryGetAppServicePlan(client, nonNull_1.nonNullProp(plan, 'resourceGroup'), nonNull_1.nonNullProp(plan, 'name'));
                    isPlanLinux = !!(epPlan === null || epPlan === void 0 ? void 0 : epPlan.reserved);
                }
                // plan.kind will contain "linux" for Linux plans, but will _not_ contain "windows" for Windows plans. Thus we check "isLinux" for both cases
                if (isNewSiteLinux === isPlanLinux) {
                    if (location && !vscode_azureextensionui_1.LocationListStep.locationMatchesName(location, plan.location)) {
                        hasFilteredLocations = true;
                    }
                    else {
                        picks.push({
                            id: plan.id,
                            label: nonNull_1.nonNullProp(plan, 'name'),
                            description: (_a = plan.sku) === null || _a === void 0 ? void 0 : _a.name,
                            data: plan
                        });
                    }
                }
            }
            if (hasFilteredLocations && location) {
                picks.push({
                    label: localize_1.localize('hasFilteredLocations', '$(warning) Only plans in the region "{0}" are shown.', location.displayName),
                    onPicked: () => { },
                    data: undefined
                });
            }
            return picks;
        });
    }
}
exports.AppServicePlanListStep = AppServicePlanListStep;
//# sourceMappingURL=AppServicePlanListStep.js.map