"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SiteNameStep = void 0;
const vscode_azureextensionui_1 = require("vscode-azureextensionui");
const localize_1 = require("../localize");
const azureClients_1 = require("../utils/azureClients");
const AppInsightsListStep_1 = require("./AppInsightsListStep");
const AppKind_1 = require("./AppKind");
const AppServicePlanListStep_1 = require("./AppServicePlanListStep");
const AppServicePlanNameStep_1 = require("./AppServicePlanNameStep");
const siteNamingRules = {
    minLength: 2,
    maxLength: 60,
    invalidCharsRegExp: /[^a-zA-Z0-9\-]/
};
class SiteNameStep extends vscode_azureextensionui_1.AzureNameStep {
    prompt(context) {
        var _a, _b;
        return __awaiter(this, void 0, void 0, function* () {
            const client = yield azureClients_1.createWebSiteClient(context);
            let placeHolder;
            if (context.environment.name === 'Azure') {
                // Unfortunately, the environment object doesn't have the url we need for this placeholder. Might be fixed in the new sdk: https://github.com/microsoft/vscode-azuretools/issues/510
                // For now, we'll only display this placeholder for the most common case
                let namePlaceholder;
                if (context.newSiteKind === AppKind_1.AppKind.functionapp) {
                    namePlaceholder = localize_1.localize('funcAppName', 'function app name');
                }
                else if ((_a = context.newSiteKind) === null || _a === void 0 ? void 0 : _a.includes(AppKind_1.AppKind.workflowapp)) {
                    namePlaceholder = localize_1.localize('logicAppName', 'logic app name');
                }
                else {
                    namePlaceholder = localize_1.localize('webAppName', 'web app name');
                }
                placeHolder = `<${namePlaceholder}>.azurewebsites.net`;
            }
            let prompt;
            if (context.newSiteKind === AppKind_1.AppKind.functionapp) {
                prompt = localize_1.localize('functionAppNamePrompt', 'Enter a globally unique name for the new function app.');
            }
            else if ((_b = context.newSiteKind) === null || _b === void 0 ? void 0 : _b.includes(AppKind_1.AppKind.workflowapp)) {
                prompt = localize_1.localize('functionAppNamePrompt', 'Enter a globally unique name for the new logic app.');
            }
            else {
                prompt = localize_1.localize('webAppNamePrompt', 'Enter a globally unique name for the new web app.');
            }
            context.newSiteName = (yield context.ui.showInputBox({
                prompt,
                placeHolder,
                validateInput: (name) => __awaiter(this, void 0, void 0, function* () { return yield this.validateSiteName(client, name); })
            })).trim();
            context.valuesToMask.push(context.newSiteName);
            const namingRules = [vscode_azureextensionui_1.resourceGroupNamingRules];
            if (context.newSiteKind === AppKind_1.AppKind.functionapp) {
                namingRules.push(vscode_azureextensionui_1.storageAccountNamingRules);
            }
            else {
                namingRules.push(AppServicePlanNameStep_1.appServicePlanNamingRules);
            }
            namingRules.push(AppInsightsListStep_1.appInsightsNamingRules);
            context.relatedNameTask = this.generateRelatedName(context, context.newSiteName, namingRules);
        });
    }
    getRelatedName(context, name) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.generateRelatedName(context, name, AppServicePlanNameStep_1.appServicePlanNamingRules);
        });
    }
    shouldPrompt(context) {
        return !context.newSiteName;
    }
    isRelatedNameAvailable(context, name) {
        return __awaiter(this, void 0, void 0, function* () {
            const tasks = [vscode_azureextensionui_1.ResourceGroupListStep.isNameAvailable(context, name)];
            if (context.newSiteKind === AppKind_1.AppKind.functionapp) {
                tasks.push(vscode_azureextensionui_1.StorageAccountListStep.isNameAvailable(context, name));
            }
            else {
                tasks.push(AppServicePlanListStep_1.AppServicePlanListStep.isNameAvailable(context, name, name));
            }
            return (yield Promise.all(tasks)).every((v) => v);
        });
    }
    validateSiteName(client, name) {
        return __awaiter(this, void 0, void 0, function* () {
            name = name.trim();
            if (name.length < siteNamingRules.minLength || name.length > siteNamingRules.maxLength) {
                return localize_1.localize('invalidLength', 'The name must be between {0} and {1} characters.', siteNamingRules.minLength, siteNamingRules.maxLength);
            }
            else if (siteNamingRules.invalidCharsRegExp.test(name)) {
                return localize_1.localize('invalidChars', "The name can only contain letters, numbers, or hyphens.");
            }
            else {
                const nameAvailability = yield client.checkNameAvailability(name, 'Site');
                if (!nameAvailability.nameAvailable) {
                    return nameAvailability.message;
                }
                else {
                    return undefined;
                }
            }
        });
    }
}
exports.SiteNameStep = SiteNameStep;
//# sourceMappingURL=SiteNameStep.js.map