"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createKuduClient = void 0;
const ms_rest_js_1 = require("@azure/ms-rest-js");
const vscode_azureextensionui_1 = require("vscode-azureextensionui");
const localize_1 = require("./localize");
const nonNull_1 = require("./utils/nonNull");
/**
 * Our first preference is to use the subscription's token credentials, but some types of sites only work with the basic publishing credentials
 * We'll ping the site, catch an 'Unauthorized' error, get the publishing creds, ping with those creds, and use them going forward if they work
 * Finally, we'll cache the client on the action context to avoid re-doing these requests multiple times for one action
 */
function createKuduClient(context, site) {
    return __awaiter(this, void 0, void 0, function* () {
        if (!context._cachedKuduClient) {
            if (!site.kuduHostName) {
                throw new Error(localize_1.localize('notSupportedLinux', 'This operation is not supported by this app service plan.'));
            }
            const clientOptions = { baseUri: site.kuduUrl, userAgent: vscode_azureextensionui_1.appendExtensionUserAgent };
            let credentials = site.subscription.credentials;
            try {
                yield pingKuduSite(context, site, site.subscription.credentials);
            }
            catch (error) {
                if (vscode_azureextensionui_1.parseError(error).errorType.toLowerCase() === '401') {
                    try {
                        const client = yield site.createClient(context);
                        const user = yield client.getWebAppPublishCredential();
                        const basicCreds = new ms_rest_js_1.BasicAuthenticationCredentials(nonNull_1.nonNullProp(user, 'publishingUserName'), nonNull_1.nonNullProp(user, 'publishingPassword'));
                        yield pingKuduSite(context, site, basicCreds);
                        credentials = basicCreds;
                        context.telemetry.properties.usedPublishCreds = 'true';
                    }
                    catch (pubCredError) {
                        // Pub creds didn't work. Fall back to original credentials
                        context.telemetry.properties.pubCredError = vscode_azureextensionui_1.parseError(pubCredError).message;
                    }
                }
            }
            context._cachedKuduClient = new (yield Promise.resolve().then(() => require('vscode-azurekudu'))).KuduClient(credentials, clientOptions);
        }
        return context._cachedKuduClient;
    });
}
exports.createKuduClient = createKuduClient;
function pingKuduSite(context, site, credentials) {
    return __awaiter(this, void 0, void 0, function* () {
        const client = yield vscode_azureextensionui_1.createGenericClient(context, credentials);
        yield client.sendRequest({ method: 'HEAD', url: site.kuduUrl });
    });
}
//# sourceMappingURL=createKuduClient.js.map