"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createSlot = void 0;
const vscode_1 = require("vscode");
const getNewFileShareName_1 = require("./createAppService/getNewFileShareName");
const extensionVariables_1 = require("./extensionVariables");
const localize_1 = require("./localize");
const azureClients_1 = require("./utils/azureClients");
function createSlot(site, existingSlots, context) {
    return __awaiter(this, void 0, void 0, function* () {
        const client = yield azureClients_1.createWebSiteClient([context, site.subscription]);
        const slotName = (yield context.ui.showInputBox({
            prompt: localize_1.localize('enterSlotName', 'Enter a unique name for the new deployment slot'),
            stepName: 'slotName',
            validateInput: (value) => __awaiter(this, void 0, void 0, function* () { return validateSlotName(value, client, site); })
        })).trim();
        const newDeploymentSlot = {
            name: slotName,
            kind: site.kind,
            location: site.location,
            serverFarmId: site.serverFarmId,
            siteConfig: {
                appSettings: [] // neccesary to have clean appSettings; by default it copies the production's slot
            }
        };
        const configurationSource = yield chooseConfigurationSource(context, site, existingSlots);
        if (configurationSource) {
            const appSettings = yield parseAppSettings(context, configurationSource);
            // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
            newDeploymentSlot.siteConfig.appSettings = appSettings;
        }
        context.showCreatingTreeItem(slotName);
        const creatingSlot = localize_1.localize('creatingSlot', 'Creating slot "{0}"...', slotName);
        extensionVariables_1.ext.outputChannel.appendLog(creatingSlot);
        return yield vscode_1.window.withProgress({ location: vscode_1.ProgressLocation.Notification, title: creatingSlot }, () => __awaiter(this, void 0, void 0, function* () {
            return yield client.webApps.createOrUpdateSlot(site.resourceGroup, site.siteName, newDeploymentSlot, slotName);
        }));
    });
}
exports.createSlot = createSlot;
const slotNamingRules = {
    minLength: 2,
    maxLength: 59,
    invalidCharsRegExp: /[^a-zA-Z0-9\-]/
};
function validateSlotName(value, client, site) {
    return __awaiter(this, void 0, void 0, function* () {
        value = value.trim();
        // Can not have "production" as a slot name, but checkNameAvailability doesn't validate that
        if (value === 'production') {
            return localize_1.localize('slotNotAvailable', 'The slot name "{0}" is not available.', value);
        }
        else if (value.length < slotNamingRules.minLength) {
            return localize_1.localize('nameTooShort', 'The slot name must be at least {0} characters.', slotNamingRules.minLength);
        }
        else if (value.length + site.siteName.length > slotNamingRules.maxLength) {
            return localize_1.localize('nameTooLong', 'The combined site name and slot name must be fewer than {0} characters.', slotNamingRules.maxLength);
        }
        else if (slotNamingRules.invalidCharsRegExp.test(value)) {
            return localize_1.localize('invalidChars', "The name can only contain letters, numbers, or hyphens.");
        }
        else {
            const nameAvailability = yield client.checkNameAvailability(`${site.siteName}-${value}`, 'Slot');
            if (!nameAvailability.nameAvailable) {
                return nameAvailability.message;
            }
            else {
                return undefined;
            }
        }
    });
}
function chooseConfigurationSource(context, site, existingSlots) {
    return __awaiter(this, void 0, void 0, function* () {
        if (site.isFunctionApp) {
            // Function apps always clone from production slot without prompting
            return site;
        }
        else {
            const configurationSources = [{
                    label: localize_1.localize('dontClone', "Don't clone configuration from an existing slot"),
                    data: undefined
                }];
            // add the production slot itself
            configurationSources.push({
                label: site.fullName,
                data: site
            });
            // add the web app's current deployment slots
            for (const slot of existingSlots) {
                configurationSources.push({
                    label: slot.fullName,
                    data: slot
                });
            }
            const placeHolder = localize_1.localize('chooseSource', 'Choose a configuration source.');
            return (yield context.ui.showQuickPick(configurationSources, { placeHolder, stepName: 'slotConfigSource' })).data;
        }
    });
}
function parseAppSettings(context, site) {
    return __awaiter(this, void 0, void 0, function* () {
        const client = yield site.createClient(context);
        const appSettings = yield client.listApplicationSettings();
        const appSettingPairs = [];
        if (appSettings.properties) {
            // iterate String Dictionary to parse into NameValuePair[]
            for (const key of Object.keys(appSettings.properties)) {
                let value = appSettings.properties[key];
                // This has to be different when cloning configuration for a function app slot
                if (site.isFunctionApp && key === 'WEBSITE_CONTENTSHARE') {
                    value = getNewFileShareName_1.getNewFileShareName(site.fullName);
                }
                appSettingPairs.push({ name: key, value });
            }
        }
        return appSettingPairs;
    });
}
//# sourceMappingURL=createSlot.js.map