"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.deploy = void 0;
const fse = require("fs-extra");
const path = require("path");
const vscode_1 = require("vscode");
const extensionVariables_1 = require("../extensionVariables");
const localize_1 = require("../localize");
const ScmType_1 = require("../ScmType");
const randomUtils_1 = require("../utils/randomUtils");
const deployToStorageAccount_1 = require("./deployToStorageAccount");
const deployWar_1 = require("./deployWar");
const deployZip_1 = require("./deployZip");
const localGitDeploy_1 = require("./localGitDeploy");
const runDeployTask_1 = require("./runDeployTask");
const syncTriggersPostDeploy_1 = require("./syncTriggersPostDeploy");
/**
 * NOTE: This leverages a command with id `ext.prefix + '.showOutputChannel'` that should be registered by each extension
 */
function deploy(site, fsPath, context) {
    return __awaiter(this, void 0, void 0, function* () {
        const client = yield site.createClient(context);
        const config = yield client.getSiteConfig();
        // We use the AppServicePlan in a few places, but we don't want to delay deployment, so start the promise now and save as a const
        const aspPromise = client.getAppServicePlan();
        try {
            context.telemetry.properties.sourceHash = randomUtils_1.randomUtils.getPseudononymousStringHash(fsPath);
            context.telemetry.properties.destHash = randomUtils_1.randomUtils.getPseudononymousStringHash(site.fullName);
            context.telemetry.properties.scmType = String(config.scmType);
            context.telemetry.properties.isSlot = site.isSlot ? 'true' : 'false';
            context.telemetry.properties.alwaysOn = config.alwaysOn ? 'true' : 'false';
            context.telemetry.properties.linuxFxVersion = getLinuxFxVersionForTelemetry(config);
            context.telemetry.properties.nodeVersion = String(config.nodeVersion);
            context.telemetry.properties.pythonVersion = String(config.pythonVersion);
            context.telemetry.properties.hasCors = config.cors ? 'true' : 'false';
            context.telemetry.properties.hasIpSecurityRestrictions = config.ipSecurityRestrictions && config.ipSecurityRestrictions.length > 0 ? 'true' : 'false';
            context.telemetry.properties.javaVersion = String(config.javaVersion);
            context.telemetry.properties.siteKind = site.kind;
            client.getState().then((state) => {
                context.telemetry.properties.state = state;
            }, () => {
                // ignore
            });
            aspPromise.then((plan) => {
                if (plan) {
                    context.telemetry.properties.planStatus = String(plan.status);
                    context.telemetry.properties.planKind = String(plan.kind);
                    if (plan.sku) {
                        context.telemetry.properties.planSize = String(plan.sku.size);
                        context.telemetry.properties.planTier = String(plan.sku.tier);
                    }
                }
            }, () => {
                // ignore
            });
        }
        catch (error) {
            // Ignore
        }
        const title = localize_1.localize('deploying', 'Deploying to "{0}"... Check [output window](command:{1}) for status.', site.fullName, extensionVariables_1.ext.prefix + '.showOutputChannel');
        yield vscode_1.window.withProgress({ location: vscode_1.ProgressLocation.Notification, title }, () => __awaiter(this, void 0, void 0, function* () {
            if (context.stopAppBeforeDeploy) {
                extensionVariables_1.ext.outputChannel.appendLog(localize_1.localize('stoppingApp', 'Stopping app...'), { resourceName: site.fullName });
                yield client.stop();
            }
            extensionVariables_1.ext.outputChannel.appendLog(localize_1.localize('deployStart', 'Starting deployment...'), { resourceName: site.fullName });
            try {
                if (!context.deployMethod && config.scmType === ScmType_1.ScmType.GitHub) {
                    throw new Error(localize_1.localize('gitHubConnected', '"{0}" is connected to a GitHub repository. Push to GitHub repository to deploy.', site.fullName));
                }
                else if (!context.deployMethod && config.scmType === ScmType_1.ScmType.LocalGit) {
                    yield localGitDeploy_1.localGitDeploy(site, { fsPath: fsPath }, context);
                }
                else {
                    if (!(yield fse.pathExists(fsPath))) {
                        throw new Error(localize_1.localize('pathNotExist', 'Failed to deploy path that does not exist: {0}', fsPath));
                    }
                    const javaRuntime = site.isLinux ? config.linuxFxVersion : config.javaContainer;
                    if (javaRuntime && /^(tomcat|wildfly|jboss)/i.test(javaRuntime)) {
                        yield deployWar_1.deployWar(context, site, fsPath);
                    }
                    else if (javaRuntime && /^java/i.test(javaRuntime) && !site.isFunctionApp) {
                        const pathFileMap = new Map([
                            [path.basename(fsPath), 'app.jar']
                        ]);
                        yield deployZip_1.deployZip(context, site, fsPath, aspPromise, pathFileMap);
                    }
                    else if (context.deployMethod === 'storage') {
                        yield deployToStorageAccount_1.deployToStorageAccount(context, fsPath, site);
                    }
                    else {
                        yield deployZip_1.deployZip(context, site, fsPath, aspPromise);
                    }
                }
            }
            finally {
                if (context.stopAppBeforeDeploy) {
                    extensionVariables_1.ext.outputChannel.appendLog(localize_1.localize('startingApp', 'Starting app...'), { resourceName: site.fullName });
                    yield client.start();
                }
            }
            yield runDeployTask_1.startPostDeployTask(context, fsPath, config.scmType, site.fullName);
            if (context.syncTriggersPostDeploy) {
                // Don't sync triggers if app is stopped https://github.com/microsoft/vscode-azurefunctions/issues/1608
                const state = yield client.getState();
                if ((state === null || state === void 0 ? void 0 : state.toLowerCase()) === 'running') {
                    yield syncTriggersPostDeploy_1.syncTriggersPostDeploy(context, site);
                }
            }
        }));
    });
}
exports.deploy = deploy;
function getLinuxFxVersionForTelemetry(config) {
    const linuxFxVersion = config.linuxFxVersion || '';
    // Docker values point to the user's specific image, which we don't want to track
    return /^docker/i.test(linuxFxVersion) ? 'docker' : linuxFxVersion;
}
//# sourceMappingURL=deploy.js.map