"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.deployToStorageAccount = void 0;
const ms_rest_azure_env_1 = require("@azure/ms-rest-azure-env");
const storage_blob_1 = require("@azure/storage-blob");
const dayjs = require("dayjs");
// eslint-disable-next-line import/no-internal-modules
const relativeTime = require("dayjs/plugin/relativeTime");
// eslint-disable-next-line import/no-internal-modules
const utc = require("dayjs/plugin/utc");
const url_1 = require("url");
const vscode_azureextensionui_1 = require("vscode-azureextensionui");
const extensionVariables_1 = require("../extensionVariables");
const localize_1 = require("../localize");
const randomUtils_1 = require("../utils/randomUtils");
const runWithZipStream_1 = require("./runWithZipStream");
dayjs.extend(relativeTime);
dayjs.extend(utc);
/**
 * Method of deployment that is only intended to be used for Linux Consumption Function apps because it doesn't support kudu pushDeployment
 * To deploy with Run from Package on a Windows plan, create the app setting "WEBSITE_RUN_FROM_PACKAGE" and set it to "1".
 * Then deploy via "zipdeploy" as usual.
 */
function deployToStorageAccount(context, fsPath, site) {
    return __awaiter(this, void 0, void 0, function* () {
        context.telemetry.properties.useStorageAccountDeploy = 'true';
        const datePart = dayjs().utc().format('YYYYMMDDHHmmss');
        const randomPart = randomUtils_1.randomUtils.getRandomHexString(32);
        const blobName = `${datePart}-${randomPart}.zip`;
        const blobService = yield createBlobServiceClient(context, site);
        const blobUrl = yield createBlobFromZip(context, fsPath, site, blobService, blobName);
        const client = yield site.createClient(context);
        const appSettings = yield client.listApplicationSettings();
        appSettings.properties = appSettings.properties || {};
        delete appSettings.properties.WEBSITE_RUN_FROM_ZIP; // delete old app setting name if it exists
        appSettings.properties.WEBSITE_RUN_FROM_PACKAGE = blobUrl;
        yield client.updateApplicationSettings(appSettings);
        extensionVariables_1.ext.outputChannel.appendLog(localize_1.localize('deploymentSuccessful', 'Deployment successful.'), { resourceName: site.fullName });
        context.syncTriggersPostDeploy = true;
    });
}
exports.deployToStorageAccount = deployToStorageAccount;
function createBlobServiceClient(context, site) {
    return __awaiter(this, void 0, void 0, function* () {
        const client = yield site.createClient(context);
        // Use same storage account as AzureWebJobsStorage for deployments
        const azureWebJobsStorageKey = 'AzureWebJobsStorage';
        const settings = yield client.listApplicationSettings();
        let connectionString = settings.properties && settings.properties[azureWebJobsStorageKey];
        if (connectionString) {
            try {
                return storage_blob_1.BlobServiceClient.fromConnectionString(connectionString);
            }
            catch (error) {
                // EndpointSuffix was optional in the old sdk, but is required in the new sdk
                // https://github.com/microsoft/vscode-azurefunctions/issues/2360
                const endpointSuffix = 'EndpointSuffix';
                const separator = ';';
                if (vscode_azureextensionui_1.parseError(error).message.includes(endpointSuffix) && !connectionString.includes(endpointSuffix)) {
                    if (!connectionString.endsWith(separator)) {
                        connectionString += separator;
                    }
                    connectionString += `${endpointSuffix}=${ms_rest_azure_env_1.Environment.AzureCloud.storageEndpointSuffix}${separator}`;
                    return storage_blob_1.BlobServiceClient.fromConnectionString(connectionString);
                }
                else {
                    throw error;
                }
            }
        }
        else {
            throw new Error(localize_1.localize('azureWebJobsStorageKey', '"{0}" app setting is required for Run From Package deployment.', azureWebJobsStorageKey));
        }
    });
}
function createBlobFromZip(context, fsPath, site, blobService, blobName) {
    return __awaiter(this, void 0, void 0, function* () {
        const containerName = 'function-releases';
        const containerClient = blobService.getContainerClient(containerName);
        if (!(yield containerClient.exists())) {
            yield containerClient.create();
        }
        const blobClient = containerClient.getBlockBlobClient(blobName);
        yield runWithZipStream_1.runWithZipStream(context, {
            fsPath, site, callback: (zipStream) => __awaiter(this, void 0, void 0, function* () {
                extensionVariables_1.ext.outputChannel.appendLog(localize_1.localize('creatingBlob', 'Uploading zip package to storage container...'), { resourceName: site.fullName });
                yield blobClient.uploadStream(zipStream);
            })
        });
        // NOTE: the `result` from `uploadStream` above doesn't actually have the contentLength - thus we have to make a separate call here
        void blobClient.getProperties().then(r => {
            context.telemetry.measurements.blobSize = Number(r.contentLength);
        });
        if (blobService.credential instanceof storage_blob_1.StorageSharedKeyCredential) {
            const url = new url_1.URL(blobClient.url);
            url.search = storage_blob_1.generateBlobSASQueryParameters({
                containerName,
                blobName,
                permissions: storage_blob_1.BlobSASPermissions.parse('r'),
                startsOn: dayjs().utc().subtract(5, 'minute').toDate(),
                expiresOn: dayjs().utc().add(10, 'year').toDate()
            }, blobService.credential).toString();
            return url.toString();
        }
        else {
            throw new Error('Internal Error: Expected credential to be of type "StorageSharedKeyCredential".');
        }
    });
}
//# sourceMappingURL=deployToStorageAccount.js.map