"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getDeployFsPath = void 0;
const fse = require("fs-extra");
const path = require("path");
const vscode = require("vscode");
const vscode_azureextensionui_1 = require("vscode-azureextensionui");
const extensionVariables_1 = require("../extensionVariables");
const localize_1 = require("../localize");
const pathUtils_1 = require("../utils/pathUtils");
const settings_1 = require("../utils/settings");
const workspaceUtil = require("../utils/workspace");
const deploySubpathSetting = 'deploySubpath';
/**
 * Entry point can be the workspace folder, the fsPath, or the tree item being deployed to
 * In App Service, users can deploy specific artifact files (such as .jar) which is handled by selectWorkspaceFile
 * This enforces that the `effectiveDeployFsPath` is currently open in a workspace
 */
function getDeployFsPath(context, target, fileExtensions) {
    return __awaiter(this, void 0, void 0, function* () {
        let originalDeployFsPath;
        let effectiveDeployFsPath;
        let workspaceFolder;
        if (target instanceof vscode.Uri) {
            originalDeployFsPath = target.fsPath;
            workspaceFolder = vscode.workspace.getWorkspaceFolder(target);
            effectiveDeployFsPath = yield appendDeploySubpathSetting(context, workspaceFolder, target.fsPath);
        }
        else if (typeof target === 'string') {
            originalDeployFsPath = target;
            workspaceFolder = vscode.workspace.getWorkspaceFolder(vscode.Uri.file(target));
            effectiveDeployFsPath = yield appendDeploySubpathSetting(context, workspaceFolder, target);
        }
        else if (vscode.workspace.workspaceFolders && vscode.workspace.workspaceFolders.length === 1) {
            // If there is only one workspace and it has 'deploySubPath' set - return that value without prompting
            const singleWorkspace = vscode.workspace.workspaceFolders[0];
            const deploySubpath = settings_1.getWorkspaceSetting(deploySubpathSetting, extensionVariables_1.ext.prefix, singleWorkspace);
            if (deploySubpath) {
                context.telemetry.properties.hasDeploySubpathSetting = 'true';
                originalDeployFsPath = singleWorkspace.uri.fsPath;
                effectiveDeployFsPath = path.join(singleWorkspace.uri.fsPath, deploySubpath);
                workspaceFolder = singleWorkspace;
            }
        }
        if (!originalDeployFsPath || !effectiveDeployFsPath) {
            if (typeof fileExtensions === 'string') {
                fileExtensions = [fileExtensions];
            }
            const selectFile = localize_1.localize('selectDeployFile', 'Select the {0} file to deploy', fileExtensions ? fileExtensions.join('/') : '');
            const selectFolder = localize_1.localize('selectDeployFolder', 'Select the folder to deploy');
            const selectedItem = fileExtensions ?
                yield workspaceUtil.selectWorkspaceFile(context, selectFile, fileExtensions) :
                yield workspaceUtil.selectWorkspaceFolder(context, selectFolder);
            if (selectedItem instanceof vscode.Uri) {
                originalDeployFsPath = selectedItem.fsPath;
                workspaceFolder = vscode.workspace.getWorkspaceFolder(selectedItem);
            }
            else {
                originalDeployFsPath = selectedItem.uri.fsPath;
                workspaceFolder = selectedItem;
            }
            effectiveDeployFsPath = yield appendDeploySubpathSetting(context, workspaceFolder, originalDeployFsPath);
        }
        void addRuntimeFileTelemetry(context, effectiveDeployFsPath);
        if (!workspaceFolder) {
            promptToOpenWorkspace(context, originalDeployFsPath);
        }
        context.telemetry.properties.deployingSubpathOfWorkspace = String(pathUtils_1.isSubpath(workspaceFolder.uri.fsPath, effectiveDeployFsPath));
        return { originalDeployFsPath, effectiveDeployFsPath, workspaceFolder };
    });
}
exports.getDeployFsPath = getDeployFsPath;
function addRuntimeFileTelemetry(context, effectiveDeployFsPath) {
    return __awaiter(this, void 0, void 0, function* () {
        const runtimeFiles = [];
        const tasks = [
            ...['package.json', 'requirements.txt', 'pom.xml'].map(f => checkRuntimeFile(runtimeFiles, effectiveDeployFsPath, f)),
            ...['dll', 'jar', 'war', 'csproj', 'fsproj'].map(e => checkRuntimeExtension(runtimeFiles, effectiveDeployFsPath, e))
        ];
        yield Promise.all(tasks);
        context.telemetry.properties.runtimeFiles = runtimeFiles.sort().join('|');
    });
}
function checkRuntimeFile(runtimeFiles, effectiveDeployFsPath, fileName) {
    return __awaiter(this, void 0, void 0, function* () {
        if (yield fse.pathExists(path.join(effectiveDeployFsPath, fileName))) {
            runtimeFiles.push(fileName);
        }
    });
}
function checkRuntimeExtension(runtimeFiles, effectiveDeployFsPath, extension) {
    return __awaiter(this, void 0, void 0, function* () {
        const files = yield vscode.workspace.findFiles(new vscode.RelativePattern(effectiveDeployFsPath, `*.${extension}`), undefined, 1 /* maxResults */);
        if (files.length > 0) {
            runtimeFiles.push(extension);
        }
    });
}
/**
 * Appends the deploySubpath setting if the target path matches the root of a workspace folder
 * If the targetPath is a sub folder instead of the root, overwrites with the subpath setting and warns the user
 */
function appendDeploySubpathSetting(context, workspaceFolder, targetPath) {
    return __awaiter(this, void 0, void 0, function* () {
        if (workspaceFolder) {
            const deploySubPath = settings_1.getWorkspaceSetting(deploySubpathSetting, extensionVariables_1.ext.prefix, workspaceFolder);
            if (deploySubPath) {
                context.telemetry.properties.hasDeploySubpathSetting = 'true';
                if (pathUtils_1.isPathEqual(workspaceFolder.uri.fsPath, targetPath)) {
                    return path.join(targetPath, deploySubPath);
                }
                else {
                    const fsPathWithSetting = path.join(workspaceFolder.uri.fsPath, deploySubPath);
                    if (!pathUtils_1.isPathEqual(fsPathWithSetting, targetPath)) {
                        context.telemetry.properties.overwriteTargetWithSubpathSetting = 'true';
                        const settingKey = 'showDeploySubpathWarning';
                        if (settings_1.getWorkspaceSetting(settingKey, extensionVariables_1.ext.prefix)) {
                            const selectedFolder = path.relative(workspaceFolder.uri.fsPath, targetPath);
                            const message = localize_1.localize('mismatchDeployPath', 'Deploying "{0}" instead of selected folder "{1}". Use "{2}.{3}" to change this behavior.', deploySubPath, selectedFolder, extensionVariables_1.ext.prefix, deploySubpathSetting);
                            // don't wait
                            void context.ui.showWarningMessage(message, { title: localize_1.localize('ok', 'OK') }, vscode_azureextensionui_1.DialogResponses.dontWarnAgain).then((result) => __awaiter(this, void 0, void 0, function* () {
                                if (result === vscode_azureextensionui_1.DialogResponses.dontWarnAgain) {
                                    yield settings_1.updateGlobalSetting(settingKey, false, extensionVariables_1.ext.prefix);
                                }
                            }));
                        }
                    }
                    return fsPathWithSetting;
                }
            }
        }
        return targetPath;
    });
}
function promptToOpenWorkspace(context, originalDeployFsPath) {
    const openInNewWindow = { title: localize_1.localize('openInNewWindow', 'Open in new window') };
    const message = localize_1.localize('folderOpenWarning', 'Failed to deploy because "{0}" is not part of an open workspace.', path.basename(originalDeployFsPath));
    // don't wait
    void context.ui.showWarningMessage(message, openInNewWindow).then((result) => __awaiter(this, void 0, void 0, function* () {
        yield vscode_azureextensionui_1.callWithTelemetryAndErrorHandling('deployWarning.openInNewWindow', (postDeployContext) => __awaiter(this, void 0, void 0, function* () {
            postDeployContext.telemetry.properties.dialogResult = result === null || result === void 0 ? void 0 : result.title;
            if (result === openInNewWindow) {
                yield vscode.commands.executeCommand('vscode.openFolder', vscode.Uri.file(originalDeployFsPath), true /* forceNewWindow */);
            }
        }));
    }));
    throw new vscode_azureextensionui_1.UserCancelledError('openInNewWindow');
}
//# sourceMappingURL=getDeployFsPath.js.map