"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getDeployNode = void 0;
const vscode_azureextensionui_1 = require("vscode-azureextensionui");
const extensionVariables_1 = require("../extensionVariables");
const localize_1 = require("../localize");
const settings_1 = require("../utils/settings");
const IDeployContext_1 = require("./IDeployContext");
/**
 * Converts the args passed in by VS Code and any relevant settings into the node used to deploy
 *
 * @param arg1 The first arg passed in by VS Code to the deploy command. Typically the node or uri
 * @param arg2 The second arg passed in by VS Code to the deploy command. Usually this is ignored, but can be the appId if called programatically from an API
 */
function getDeployNode(context, tree, arg1, arg2, expectedContextValue) {
    return __awaiter(this, void 0, void 0, function* () {
        let node;
        if (arg1 instanceof vscode_azureextensionui_1.AzExtTreeItem) {
            node = arg1;
            context.appSource = IDeployContext_1.AppSource.tree;
        }
        else if (typeof arg2 === 'string' && arg2) {
            node = yield tree.findTreeItem(arg2, context);
            if (!node) {
                throw new Error(localize_1.localize('noMatchingApp', 'Failed to find app matching id "{0}".', arg2));
            }
            context.appSource = IDeployContext_1.AppSource.api;
        }
        else {
            const defaultAppId = settings_1.getWorkspaceSetting(context.defaultAppSetting, extensionVariables_1.ext.prefix, context.workspaceFolder.uri.fsPath);
            if (defaultAppId && defaultAppId.toLowerCase() !== 'none') {
                node = yield tree.findTreeItem(defaultAppId, context);
                if (node) {
                    context.appSource = IDeployContext_1.AppSource.setting;
                }
                else {
                    extensionVariables_1.ext.outputChannel.appendLog(localize_1.localize('appFromSettingNotFound', 'WARNING: Failed to find app matching setting "{0}.{1}" with id "{2}"', extensionVariables_1.ext.prefix, context.defaultAppSetting, defaultAppId));
                }
            }
            if (!node) {
                const newNodes = [];
                const disposable = tree.onTreeItemCreate(newNode => { newNodes.push(newNode); });
                try {
                    node = yield tree.showTreeItemPicker(expectedContextValue, context);
                }
                finally {
                    disposable.dispose();
                }
                context.isNewApp = newNodes.some(newNode => node && newNode.fullId === node.fullId);
                context.appSource = IDeployContext_1.AppSource.nodePicker;
            }
        }
        context.telemetry.properties.appSource = context.appSource;
        context.telemetry.properties.isNewApp = String(!!context.isNewApp);
        return node;
    });
}
exports.getDeployNode = getDeployNode;
//# sourceMappingURL=getDeployNode.js.map