"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.localGitDeploy = void 0;
// eslint-disable-next-line import/no-internal-modules
const git = require("simple-git/promise");
const vscode = require("vscode");
const vscode_azureextensionui_1 = require("vscode-azureextensionui");
const extensionVariables_1 = require("../extensionVariables");
const localize_1 = require("../localize");
const nonNull_1 = require("../utils/nonNull");
const openUrl_1 = require("../utils/openUrl");
const verifyNoRunFromPackageSetting_1 = require("../verifyNoRunFromPackageSetting");
const waitForDeploymentToComplete_1 = require("./waitForDeploymentToComplete");
function localGitDeploy(site, options, context) {
    return __awaiter(this, void 0, void 0, function* () {
        const client = yield site.createClient(context);
        const publishCredentials = yield client.getWebAppPublishCredential();
        const publishingPassword = nonNull_1.nonNullProp(publishCredentials, 'publishingPassword');
        const publishingUserName = nonNull_1.nonNullProp(publishCredentials, 'publishingUserName');
        yield vscode_azureextensionui_1.callWithMaskHandling(() => __awaiter(this, void 0, void 0, function* () {
            const remote = `https://${encodeURIComponent(publishingUserName)}:${encodeURIComponent(publishingPassword)}@${site.gitUrl}`;
            const localGit = git(options.fsPath);
            let status;
            try {
                status = yield localGit.status();
                if (status.files.length > 0 && !options.commit) {
                    const message = localize_1.localize('localGitUncommit', '{0} uncommitted change(s) in local repo "{1}"', status.files.length, options.fsPath);
                    const deployAnyway = { title: localize_1.localize('deployAnyway', 'Deploy Anyway') };
                    yield context.ui.showWarningMessage(message, { modal: true, stepName: 'pushWithUncommitChanges' }, deployAnyway);
                    context.telemetry.properties.pushWithUncommitChanges = 'true';
                }
                yield verifyNoRunFromPackageSetting_1.verifyNoRunFromPackageSetting(context, site);
                extensionVariables_1.ext.outputChannel.appendLog(localize_1.localize('localGitDeploy', `Deploying Local Git repository to "${site.fullName}"...`), { resourceName: site.fullName });
                yield tryPushAndWaitForDeploymentToComplete();
            }
            catch (err) {
                // eslint-disable-next-line @typescript-eslint/no-unsafe-member-access, @typescript-eslint/no-unsafe-call
                if (err.message.indexOf('spawn git ENOENT') >= 0) {
                    const installString = localize_1.localize('Install', 'Install');
                    const input = yield vscode.window.showErrorMessage(localize_1.localize('GitRequired', 'Git must be installed to use Local Git Deploy.'), installString);
                    if (input === installString) {
                        yield openUrl_1.openUrl('https://git-scm.com/downloads');
                    }
                    context.telemetry.properties.gitNotInstalled = 'true';
                    return undefined;
                    // eslint-disable-next-line @typescript-eslint/no-unsafe-member-access, @typescript-eslint/no-unsafe-call
                }
                else if (err.message.indexOf('error: failed to push') >= 0) {
                    const forcePushMessage = { title: localize_1.localize('forcePush', 'Force Push') };
                    const pushReject = localize_1.localize('localGitPush', 'Push rejected due to Git history diverging.');
                    yield context.ui.showWarningMessage(pushReject, { modal: true, stepName: 'forcePush' }, forcePushMessage);
                    context.telemetry.properties.forcePush = 'true';
                    yield tryPushAndWaitForDeploymentToComplete(true);
                }
                else {
                    throw err;
                }
            }
            function tryPushAndWaitForDeploymentToComplete(forcePush = false) {
                return __awaiter(this, void 0, void 0, function* () {
                    const tokenSource = new vscode.CancellationTokenSource();
                    const token = tokenSource.token;
                    try {
                        if (options.commit) {
                            const commitOptions = { '-a': null };
                            yield localGit.commit('Deployed via Azure App Service Extension', undefined, commitOptions);
                        }
                        const commitId = (yield localGit.log()).latest.hash;
                        yield new Promise((resolve, reject) => {
                            var _a;
                            const pushOptions = forcePush ? { '-f': null } : {};
                            localGit.push(remote, `HEAD:${(_a = options.branch) !== null && _a !== void 0 ? _a : 'master'}`, pushOptions).catch((error) => {
                                reject(error);
                                tokenSource.cancel();
                            });
                            waitForDeploymentToComplete_1.waitForDeploymentToComplete(context, site, commitId, token).then(resolve).catch(reject);
                        });
                    }
                    finally {
                        tokenSource.dispose();
                    }
                });
            }
        }), publishingPassword);
    });
}
exports.localGitDeploy = localGitDeploy;
//# sourceMappingURL=localGitDeploy.js.map