"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.handleFailedPreDeployTask = exports.startPostDeployTask = exports.tryRunPreDeployTask = exports.runPreDeployTask = void 0;
const vscode = require("vscode");
const vscode_azureextensionui_1 = require("vscode-azureextensionui");
const extensionVariables_1 = require("../extensionVariables");
const localize_1 = require("../localize");
const ScmType_1 = require("../ScmType");
const taskUtils_1 = require("../utils/taskUtils");
function runPreDeployTask(context, deployFsPath, scmType) {
    return __awaiter(this, void 0, void 0, function* () {
        const preDeployResult = yield tryRunPreDeployTask(context, deployFsPath, scmType);
        if (preDeployResult.failedToFindTask) {
            throw new Error(`Failed to find pre-deploy task "${preDeployResult.taskName}". Modify your tasks or the setting "${extensionVariables_1.ext.prefix}.preDeployTask".`);
        }
        else if (preDeployResult.exitCode !== undefined && preDeployResult.exitCode !== 0) {
            yield handleFailedPreDeployTask(context, preDeployResult);
        }
    });
}
exports.runPreDeployTask = runPreDeployTask;
function tryRunPreDeployTask(context, deployFsPath, scmType) {
    return __awaiter(this, void 0, void 0, function* () {
        const settingKey = 'preDeployTask';
        const taskName = vscode.workspace.getConfiguration(extensionVariables_1.ext.prefix, vscode.Uri.file(deployFsPath)).get(settingKey);
        context.telemetry.properties.hasPreDeployTask = String(!!taskName);
        let preDeployTaskResult = { taskName, exitCode: undefined, failedToFindTask: false };
        if (taskName && shouldExecuteTask(context, scmType, settingKey, taskName)) {
            const task = yield taskUtils_1.taskUtils.findTask(deployFsPath, taskName);
            context.telemetry.properties.foundPreDeployTask = String(!!task);
            if (task) {
                const progressMessage = localize_1.localize('runningTask', 'Running preDeployTask "{0}"...', taskName);
                yield vscode.window.withProgress({ location: vscode.ProgressLocation.Notification, title: progressMessage }, () => __awaiter(this, void 0, void 0, function* () {
                    yield taskUtils_1.taskUtils.executeIfNotActive(task);
                    preDeployTaskResult = yield waitForPreDeployTask(task, deployFsPath);
                    context.telemetry.properties.preDeployTaskExitCode = String(preDeployTaskResult.exitCode);
                }));
            }
            else {
                preDeployTaskResult.failedToFindTask = true;
            }
        }
        return preDeployTaskResult;
    });
}
exports.tryRunPreDeployTask = tryRunPreDeployTask;
/**
 * Starts the post deploy task, but doesn't wait for the result (not worth it)
 */
function startPostDeployTask(context, deployFsPath, scmType, resourceName) {
    return __awaiter(this, void 0, void 0, function* () {
        const settingKey = 'postDeployTask';
        const taskName = vscode.workspace.getConfiguration(extensionVariables_1.ext.prefix, vscode.Uri.file(deployFsPath)).get(settingKey);
        context.telemetry.properties.hasPostDeployTask = String(!!taskName);
        if (taskName && shouldExecuteTask(context, scmType, settingKey, taskName)) {
            const task = yield taskUtils_1.taskUtils.findTask(deployFsPath, taskName);
            context.telemetry.properties.foundPostDeployTask = String(!!task);
            if (task) {
                yield taskUtils_1.taskUtils.executeIfNotActive(task);
                extensionVariables_1.ext.outputChannel.appendLog(localize_1.localize('startedPostDeployTask', 'Started {0} "{1}".', settingKey, taskName), { resourceName });
            }
            else {
                extensionVariables_1.ext.outputChannel.appendLog(localize_1.localize('noPostDeployTask', 'WARNING: Failed to find {0} "{1}".', settingKey, taskName), { resourceName });
            }
        }
    });
}
exports.startPostDeployTask = startPostDeployTask;
function shouldExecuteTask(context, scmType, settingKey, taskName) {
    // We don't run deploy tasks for non-zipdeploy since that stuff should be handled by kudu
    const shouldExecute = context.deployMethod === 'storage' || context.deployMethod === 'zip' || (scmType !== ScmType_1.ScmType.LocalGit && scmType !== ScmType_1.ScmType.GitHub);
    if (!shouldExecute) {
        extensionVariables_1.ext.outputChannel.appendLog(localize_1.localize('ignoringDeployTask', 'WARNING: Ignoring {0} "{1}" for non-zip deploy.', settingKey, taskName));
    }
    return shouldExecute;
}
function waitForPreDeployTask(preDeployTask, deployFsPath) {
    return __awaiter(this, void 0, void 0, function* () {
        return yield new Promise((resolve) => {
            const errorListener = vscode.tasks.onDidEndTaskProcess((e) => {
                if (taskUtils_1.taskUtils.isTaskInScopeOfPath(e.execution.task, deployFsPath) && e.exitCode !== 0) {
                    // Throw if _any_ task fails since preDeployTasks can depend on other tasks)
                    errorListener.dispose();
                    resolve({ taskName: e.execution.task.name, exitCode: e.exitCode, failedToFindTask: false });
                }
                // this is the actual preDeployTask that we are waiting on
                if (taskUtils_1.taskUtils.isTaskEqual(e.execution.task, preDeployTask)) {
                    errorListener.dispose();
                    resolve({ taskName: e.execution.task.name, exitCode: e.exitCode, failedToFindTask: false });
                }
            });
        });
    });
}
function handleFailedPreDeployTask(context, preDeployResult) {
    return __awaiter(this, void 0, void 0, function* () {
        const message = localize_1.localize('taskFailed', 'Errors exist after running preDeployTask "{0}". See task output for more info.', preDeployResult.taskName);
        const deployAnyway = { title: localize_1.localize('deployAnyway', 'Deploy Anyway') };
        const openSettings = { title: localize_1.localize('openSettings', 'Open Settings') };
        const result = yield vscode.window.showErrorMessage(message, { modal: true }, deployAnyway, openSettings);
        if (result === deployAnyway) {
            context.telemetry.properties.preDeployTaskResponse = 'deployAnyway';
        }
        else if (result === openSettings) {
            context.telemetry.properties.preDeployTaskResponse = 'openSettings';
            yield vscode.commands.executeCommand('workbench.action.openSettings');
            throw new vscode_azureextensionui_1.UserCancelledError('preDeployFailed|OpenSettings');
        }
        else {
            context.telemetry.properties.preDeployTaskResponse = 'cancel';
            throw new vscode_azureextensionui_1.UserCancelledError('preDeployFailed');
        }
    });
}
exports.handleFailedPreDeployTask = handleFailedPreDeployTask;
//# sourceMappingURL=runDeployTask.js.map