"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.runWithZipStream = void 0;
const fse = require("fs-extra");
const glob_gitignore_1 = require("glob-gitignore");
const globby = require("globby");
const path = require("path");
const prettybytes = require("pretty-bytes");
const vscode = require("vscode");
const yazl = require("yazl");
const extensionVariables_1 = require("../extensionVariables");
const localize_1 = require("../localize");
const pathUtils_1 = require("../utils/pathUtils");
function runWithZipStream(context, options) {
    return __awaiter(this, void 0, void 0, function* () {
        function onFileSize(size) {
            context.telemetry.measurements.zipFileSize = size;
            extensionVariables_1.ext.outputChannel.appendLog(localize_1.localize('zipSize', 'Zip package size: {0}', prettybytes(size)), { resourceName: site.fullName });
        }
        let zipStream;
        const { site, pathFileMap, callback } = options;
        let { fsPath } = options;
        if (pathUtils_1.getFileExtension(fsPath) === 'zip') {
            context.telemetry.properties.alreadyZipped = 'true';
            zipStream = fse.createReadStream(fsPath);
            // don't wait
            void fse.lstat(fsPath).then(stats => {
                onFileSize(stats.size);
            });
        }
        else {
            extensionVariables_1.ext.outputChannel.appendLog(localize_1.localize('zipCreate', 'Creating zip package...'), { resourceName: site.fullName });
            const zipFile = new yazl.ZipFile();
            let filesToZip = [];
            let sizeOfZipFile = 0;
            zipFile.outputStream.on('data', (chunk) => {
                if (typeof chunk === 'string' || Buffer.isBuffer(chunk)) {
                    sizeOfZipFile += chunk.length;
                }
            });
            zipFile.outputStream.on('finish', () => onFileSize(sizeOfZipFile));
            if ((yield fse.lstat(fsPath)).isDirectory()) {
                if (!fsPath.endsWith(path.sep)) {
                    fsPath += path.sep;
                }
                if (site.isFunctionApp) {
                    filesToZip = yield getFilesFromGitignore(fsPath, '.funcignore');
                }
                else {
                    filesToZip = yield getFilesFromGlob(fsPath, site);
                }
                for (const file of filesToZip) {
                    zipFile.addFile(path.join(fsPath, file), getPathFromMap(file, pathFileMap));
                }
            }
            else {
                zipFile.addFile(fsPath, getPathFromMap(path.basename(fsPath), pathFileMap));
            }
            zipFile.end();
            zipStream = zipFile.outputStream;
        }
        yield callback(zipStream);
    });
}
exports.runWithZipStream = runWithZipStream;
function getPathFromMap(realPath, pathfileMap) {
    return (pathfileMap === null || pathfileMap === void 0 ? void 0 : pathfileMap.get(realPath)) || realPath;
}
const commonGlobSettings = {
    dot: true,
    nodir: true,
    follow: true // Follow symlinks to get all sub folders https://github.com/microsoft/vscode-azurefunctions/issues/1289
};
/**
 * Adds files using glob filtering
 */
function getFilesFromGlob(folderPath, site) {
    return __awaiter(this, void 0, void 0, function* () {
        const zipDeployConfig = vscode.workspace.getConfiguration(extensionVariables_1.ext.prefix, vscode.Uri.file(folderPath));
        const globOptions = { cwd: folderPath, followSymbolicLinks: true, dot: true };
        const globPattern = zipDeployConfig.get('zipGlobPattern') || '**/*';
        const filesToInclude = yield globby(globPattern, globOptions);
        const zipIgnorePatternStr = 'zipIgnorePattern';
        let ignorePatternList = zipDeployConfig.get(zipIgnorePatternStr) || '';
        const filesToIgnore = yield globby(ignorePatternList, globOptions);
        if (ignorePatternList) {
            if (typeof ignorePatternList === 'string') {
                ignorePatternList = [ignorePatternList];
            }
            if (ignorePatternList.length > 0) {
                extensionVariables_1.ext.outputChannel.appendLog(localize_1.localize('zipIgnoreFileMsg', `Ignoring files from \"{0}.{1}\"`, extensionVariables_1.ext.prefix, zipIgnorePatternStr), { resourceName: site.fullName });
                for (const pattern of ignorePatternList) {
                    extensionVariables_1.ext.outputChannel.appendLine(`\"${pattern}\"`);
                }
            }
        }
        return filesToInclude.filter(file => {
            return !filesToIgnore.includes(file);
        });
    });
}
/**
 * Adds files using gitignore filtering
 */
function getFilesFromGitignore(folderPath, gitignoreName) {
    return __awaiter(this, void 0, void 0, function* () {
        let ignore = [];
        const gitignorePath = path.join(folderPath, gitignoreName);
        if (yield fse.pathExists(gitignorePath)) {
            const funcIgnoreContents = (yield fse.readFile(gitignorePath)).toString();
            ignore = funcIgnoreContents.split('\n').map(l => l.trim());
        }
        // eslint-disable-next-line @typescript-eslint/no-unsafe-call, @typescript-eslint/no-unsafe-assignment
        const paths = yield glob_gitignore_1.glob('**/*', Object.assign({ cwd: folderPath, ignore }, commonGlobSettings));
        return paths;
    });
}
//# sourceMappingURL=runWithZipStream.js.map