"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.waitForDeploymentToComplete = void 0;
const vscode_1 = require("vscode");
const vscode_azureextensionui_1 = require("vscode-azureextensionui");
const createKuduClient_1 = require("../createKuduClient");
const extensionVariables_1 = require("../extensionVariables");
const localize_1 = require("../localize");
const delay_1 = require("../utils/delay");
const kuduUtils_1 = require("../utils/kuduUtils");
const nonNull_1 = require("../utils/nonNull");
function waitForDeploymentToComplete(context, site, expectedId, token, pollingInterval = 5000) {
    return __awaiter(this, void 0, void 0, function* () {
        let fullLog = '';
        let lastLogTime = new Date(0);
        let lastErrorLine = '';
        let initialStartTime;
        let deployment;
        let permanentId;
        // a 60 second timeout period to let Kudu initialize the deployment
        const maxTimeToWaitForExpectedId = Date.now() + 60 * 1000;
        const kuduClient = yield createKuduClient_1.createKuduClient(context, site);
        while (!(token === null || token === void 0 ? void 0 : token.isCancellationRequested)) {
            [deployment, permanentId, initialStartTime] = yield tryGetLatestDeployment(context, kuduClient, permanentId, initialStartTime, expectedId);
            if ((deployment === undefined || !deployment.id)) {
                if (expectedId && Date.now() < maxTimeToWaitForExpectedId) {
                    yield delay_1.delay(pollingInterval);
                    continue;
                }
                throw new Error(localize_1.localize('failedToFindDeployment', 'Failed to get status of deployment.'));
            }
            const deploymentId = deployment.id;
            let logEntries = [];
            yield kuduUtils_1.retryKuduCall(context, 'getLogEntry', () => __awaiter(this, void 0, void 0, function* () {
                yield kuduUtils_1.ignore404Error(context, () => __awaiter(this, void 0, void 0, function* () {
                    logEntries = (yield kuduClient.deployment.getLogEntry(deploymentId)).reverse();
                }));
            }));
            let lastLogTimeForThisPoll;
            // eslint-disable-next-line no-constant-condition
            while (true) {
                const newEntry = logEntries.pop();
                if (!newEntry) {
                    break;
                }
                if (newEntry.message && newEntry.logTime && newEntry.logTime > lastLogTime) {
                    fullLog = fullLog.concat(newEntry.message);
                    extensionVariables_1.ext.outputChannel.appendLog(newEntry.message, { date: newEntry.logTime, resourceName: site.fullName });
                    lastLogTimeForThisPoll = newEntry.logTime;
                    if (/error/i.test(newEntry.message)) {
                        lastErrorLine = newEntry.message;
                    }
                }
                yield kuduUtils_1.retryKuduCall(context, 'getLogEntryDetails', () => __awaiter(this, void 0, void 0, function* () {
                    yield kuduUtils_1.ignore404Error(context, () => __awaiter(this, void 0, void 0, function* () {
                        if (newEntry.id && newEntry.detailsUrl) {
                            const details = yield kuduClient.deployment.getLogEntryDetails(deploymentId, newEntry.id);
                            logEntries.push(...cleanDetails(details));
                        }
                    }));
                }));
            }
            if (lastLogTimeForThisPoll) {
                lastLogTime = lastLogTimeForThisPoll;
            }
            if (deployment.complete) {
                if (deployment.status === 3 /* Failed */ || deployment.isTemp) { // If the deployment completed without making it to the "permanent" phase, it must have failed
                    const message = localize_1.localize('deploymentFailed', 'Deployment to "{0}" failed.', site.fullName);
                    const viewOutput = localize_1.localize('viewOutput', 'View Output');
                    // don't wait
                    void vscode_1.window.showErrorMessage(message, viewOutput).then(result => {
                        if (result === viewOutput) {
                            extensionVariables_1.ext.outputChannel.show();
                        }
                    });
                    const messageWithoutName = localize_1.localize('deploymentFailedWithoutName', 'Deployment failed.');
                    extensionVariables_1.ext.outputChannel.appendLog(messageWithoutName, { resourceName: site.fullName });
                    context.errorHandling.suppressDisplay = true;
                    // Hopefully the last line is enough to give us an idea why deployments are failing without excessively tracking everything
                    context.telemetry.properties.deployErrorLastLine = lastErrorLine;
                    throw new Error(messageWithoutName);
                }
                else {
                    context.syncTriggersPostDeploy = site.isFunctionApp && !/syncing/i.test(fullLog) && !site.isKubernetesApp && !site.isWorkflowApp;
                    return;
                }
            }
            else {
                yield delay_1.delay(pollingInterval);
            }
        }
    });
}
exports.waitForDeploymentToComplete = waitForDeploymentToComplete;
function tryGetLatestDeployment(context, kuduClient, permanentId, initialStartTime, expectedId) {
    return __awaiter(this, void 0, void 0, function* () {
        let deployment;
        if (permanentId) {
            // Use "permanentId" to find the deployment during its "permanent" phase
            deployment = yield kuduUtils_1.retryKuduCall(context, 'getResult', () => __awaiter(this, void 0, void 0, function* () {
                // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
                return yield kuduClient.deployment.getResult(permanentId);
            }));
        }
        else if (expectedId) {
            // if we have a "expectedId" we know which deployment we are looking for, so wait until latest id reflects that
            try {
                const latestDeployment = yield kuduUtils_1.retryKuduCall(context, 'getResult', () => __awaiter(this, void 0, void 0, function* () {
                    return yield kuduClient.deployment.getResult('latest');
                }));
                // if the latest deployment is temp, then a deployment has triggered so we should watch it even if it doesn't match the expectedId
                if (latestDeployment.isTemp) {
                    deployment = latestDeployment;
                }
                else if (latestDeployment.id === expectedId) {
                    deployment = latestDeployment;
                    permanentId = latestDeployment.id;
                }
            }
            catch (error) {
                const parsedError = vscode_azureextensionui_1.parseError(error);
                // swallow 404 error since "latest" might not exist on the first deployment
                if (parsedError.errorType !== '404') {
                    throw parsedError;
                }
            }
        }
        else if (initialStartTime) {
            // Use "initialReceivedTime" to find the deployment during its "temp" phase
            const deployments = yield kuduUtils_1.retryKuduCall(context, 'getDeployResults', () => __awaiter(this, void 0, void 0, function* () {
                return yield kuduClient.deployment.getDeployResults();
            }));
            deployment = deployments
                // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
                .filter(deployResult => deployResult.startTime && deployResult.startTime >= initialStartTime)
                .sort((a, b) => nonNull_1.nonNullProp(b, 'startTime').valueOf() - nonNull_1.nonNullProp(a, 'startTime').valueOf())
                .shift();
            if (deployment && !deployment.isTemp) {
                // Make note of the id once the deplyoment has shifted to the "permanent" phase, so that we can use that to find the deployment going forward
                permanentId = deployment.id;
            }
        }
        else {
            // Use "latest" to get the deployment before we know the "initialReceivedTime" or "permanentId"
            try {
                deployment = yield kuduUtils_1.retryKuduCall(context, 'getResult', () => __awaiter(this, void 0, void 0, function* () {
                    return yield kuduClient.deployment.getResult('latest');
                }));
            }
            catch (error) {
                const parsedError = vscode_azureextensionui_1.parseError(error);
                // swallow 404 error since "latest" might not exist on the first deployment
                if (parsedError.errorType !== '404') {
                    throw error;
                }
            }
            if (deployment && deployment.startTime) {
                // Make note of the startTime because that is when kudu has began the deployment process,
                // so that we can use that to find the deployment going forward
                initialStartTime = deployment.startTime;
            }
        }
        return [deployment, permanentId, initialStartTime];
    });
}
/**
 * Workaround for some weird Oryx behavior where it shows several duplicate logs on a single line
 */
function cleanDetails(entries) {
    var _a;
    const result = [];
    for (const entry of entries) {
        const oryxEOL = '\\n';
        if ((_a = entry.message) === null || _a === void 0 ? void 0 : _a.includes(oryxEOL)) {
            const newMessages = entry.message.split(oryxEOL);
            for (const newMessage of newMessages) {
                if (!entries.find(d => d.message === newMessage)) {
                    result.push(Object.assign(Object.assign({}, entry), { message: newMessage }));
                }
            }
        }
        else {
            result.push(entry);
        }
    }
    return result.reverse();
}
//# sourceMappingURL=waitForDeploymentToComplete.js.map