"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.editScmType = void 0;
const vscode_1 = require("vscode");
const vscode_azureextensionui_1 = require("vscode-azureextensionui");
const extensionVariables_1 = require("./extensionVariables");
const connectToGitHub_1 = require("./github/connectToGitHub");
const localize_1 = require("./localize");
const ScmType_1 = require("./ScmType");
const nonNull_1 = require("./utils/nonNull");
function editScmType(context, site, subscriptionContext, newScmType, showToast = true) {
    return __awaiter(this, void 0, void 0, function* () {
        const client = yield site.createClient(context);
        if (site.isLinux && (yield client.getIsConsumption(context))) {
            context.errorHandling.suppressReportIssue = true;
            throw new Error(localize_1.localize('noEditScmOnLinuxCons', 'Linux consumption plans only support zip deploy. See [here](https://aka.ms/AA7avjx) for more information.'));
        }
        const config = yield client.getSiteConfig();
        newScmType = newScmType ? newScmType : yield showScmPrompt(context, nonNull_1.nonNullProp(config, 'scmType'));
        if (newScmType === ScmType_1.ScmType.GitHub) {
            if (config.scmType !== ScmType_1.ScmType.None) {
                // GitHub cannot be configured if there is an existing configuration source-- a limitation of Azure
                yield editScmType(context, site, subscriptionContext, ScmType_1.ScmType.None, false);
            }
            yield connectToGitHub_1.connectToGitHub(site, Object.assign(context, subscriptionContext));
        }
        else {
            config.scmType = newScmType;
            // to update one property, a complete config file must be sent
            yield client.updateConfiguration(config);
        }
        if (showToast) {
            const scmTypeUpdated = localize_1.localize('deploymentSourceUpdated,', 'Deployment source for "{0}" has been updated to "{1}".', site.fullName, newScmType);
            extensionVariables_1.ext.outputChannel.appendLog(scmTypeUpdated);
            void vscode_1.window.showInformationMessage(scmTypeUpdated);
        }
        if (newScmType === ScmType_1.ScmType.LocalGit) {
            const user = yield client.getPublishingUser();
            if (user.publishingUserName) {
                // first time users must set up deployment credentials via the Portal or they will not have a UserName
                const gitCloneUri = `https://${user.publishingUserName}@${site.gitUrl}`;
                extensionVariables_1.ext.outputChannel.appendLog(localize_1.localize('gitCloneUri', 'Git Clone Uri for "{0}": "{1}"', site.fullName, gitCloneUri));
            }
        }
        // returns the updated scmType
        return newScmType;
    });
}
exports.editScmType = editScmType;
function showScmPrompt(context, currentScmType) {
    return __awaiter(this, void 0, void 0, function* () {
        const currentSource = localize_1.localize('currentSource', '(Current source)');
        const scmQuickPicks = [];
        // generate quickPicks to not include current type
        for (const key of Object.keys(ScmType_1.ScmType)) {
            const scmType = ScmType_1.ScmType[key];
            if (scmType === currentScmType) {
                // put the current source at the top of the list
                scmQuickPicks.unshift({ label: scmType, description: currentSource, data: undefined });
            }
            else {
                scmQuickPicks.push({ label: scmType, description: '', data: scmType });
            }
        }
        const options = {
            placeHolder: localize_1.localize('scmPrompt', 'Select a new source.'),
            suppressPersistence: true,
            stepName: 'editScmType'
        };
        const newScmType = (yield context.ui.showQuickPick(scmQuickPicks, options)).data;
        if (newScmType === undefined) {
            // if the user clicks the current source, treat it as a cancel
            throw new vscode_azureextensionui_1.UserCancelledError('scmTypeAlreadyMatches');
        }
        else {
            return newScmType;
        }
    });
}
//# sourceMappingURL=editScmType.js.map