"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createGitHubClient = exports.getGitHubQuickPicksWithLoadMore = exports.createQuickPickFromJsons = exports.getGitHubJsonResponse = exports.connectToGitHub = void 0;
const ms_rest_js_1 = require("@azure/ms-rest-js");
const util_1 = require("util");
const vscode = require("vscode");
const vscode_azureextensionui_1 = require("vscode-azureextensionui");
const extensionVariables_1 = require("../extensionVariables");
const localize_1 = require("../localize");
const nonNull_1 = require("../utils/nonNull");
const openUrl_1 = require("../utils/openUrl");
const verifyNoRunFromPackageSetting_1 = require("../verifyNoRunFromPackageSetting");
const GitHubBranchListStep_1 = require("./GitHubBranchListStep");
const GitHubOrgListStep_1 = require("./GitHubOrgListStep");
const GitHubRepoListStep_1 = require("./GitHubRepoListStep");
function connectToGitHub(site, context) {
    return __awaiter(this, void 0, void 0, function* () {
        const title = localize_1.localize('connectGitHubRepo', 'Connect GitHub repository');
        context.site = site;
        const wizard = new vscode_azureextensionui_1.AzureWizard(context, {
            title,
            promptSteps: [
                new GitHubOrgListStep_1.GitHubOrgListStep(),
                new GitHubRepoListStep_1.GitHubRepoListStep(),
                new GitHubBranchListStep_1.GitHubBranchListStep()
            ]
        });
        yield wizard.prompt();
        const siteSourceControl = {
            repoUrl: nonNull_1.nonNullProp(context, 'repoData').html_url,
            branch: nonNull_1.nonNullProp(context, 'branchData').name,
            isManualIntegration: false,
            deploymentRollbackEnabled: true,
            isMercurial: false
        };
        const repoName = `${nonNull_1.nonNullProp(context, 'orgData').login}/${nonNull_1.nonNullProp(context, 'repoData').name}`;
        const client = yield site.createClient(context);
        try {
            const connectingToGithub = localize_1.localize('ConnectingToGithub', '"{0}" is being connected to repo "{1}". This may take several minutes...', site.fullName, repoName);
            const connectedToGithub = localize_1.localize('ConnectedToGithub', 'Repo "{0}" is connected and deployed to "{1}".', repoName, site.fullName);
            yield vscode.window.withProgress({ location: vscode.ProgressLocation.Notification, title: connectingToGithub }, () => __awaiter(this, void 0, void 0, function* () {
                extensionVariables_1.ext.outputChannel.appendLog(connectingToGithub);
                yield verifyNoRunFromPackageSetting_1.verifyNoRunFromPackageSetting(context, site);
                yield client.updateSourceControl(siteSourceControl);
                void vscode.window.showInformationMessage(connectedToGithub);
                extensionVariables_1.ext.outputChannel.appendLog(connectedToGithub);
            }));
        }
        catch (err) {
            try {
                // a resync will fix the first broken build
                // https://github.com/projectkudu/kudu/issues/2277
                yield client.syncRepository();
            }
            catch (error) {
                const parsedError = vscode_azureextensionui_1.parseError(error);
                // The portal returns 200, but is expecting a 204 which causes it to throw an error even after a successful sync
                if (parsedError.message.indexOf('"statusCode":200') === -1) {
                    throw error;
                }
            }
        }
    });
}
exports.connectToGitHub = connectToGitHub;
function showGitHubAuthPrompt(context, site) {
    return __awaiter(this, void 0, void 0, function* () {
        const invalidToken = localize_1.localize('tokenExpired', 'Azure\'s GitHub token is invalid.  Authorize in the "Deployment Center"');
        const goToPortal = { title: localize_1.localize('goToPortal', 'Go to Portal') };
        let input = vscode_azureextensionui_1.DialogResponses.learnMore;
        while (input === vscode_azureextensionui_1.DialogResponses.learnMore) {
            input = yield vscode.window.showErrorMessage(invalidToken, { modal: true }, goToPortal, vscode_azureextensionui_1.DialogResponses.learnMore);
            if (input === vscode_azureextensionui_1.DialogResponses.learnMore) {
                context.telemetry.properties.githubLearnMore = 'true';
                yield openUrl_1.openUrl('https://aka.ms/B7g6sw');
            }
        }
        if (input === goToPortal) {
            context.telemetry.properties.githubGoToPortal = 'true';
            yield vscode_azureextensionui_1.openInPortal(context, `${site.id}/vstscd`);
        }
    });
}
function getGitHubJsonResponse(context, url) {
    return __awaiter(this, void 0, void 0, function* () {
        context.gitHubClient = context.gitHubClient || (yield createGitHubClient(context));
        // Reference for GitHub REST routes
        // https://developer.github.com/v3/
        // Note: blank after user implies look up authorized user
        try {
            const response = yield context.gitHubClient.sendRequest({ method: 'GET', url });
            const headerLink = response.headers.get('link');
            const nextLink = headerLink && parseLinkHeaderToGitHubLinkObject(headerLink).next;
            return [response.parsedBody, nextLink];
        }
        catch (error) {
            const parsedError = vscode_azureextensionui_1.parseError(error);
            if (parsedError.message.indexOf('Bad credentials') > -1) {
                // the default error is just "Bad credentials," which is an unhelpful error message
                yield showGitHubAuthPrompt(context, nonNull_1.nonNullProp(context, 'site'));
                context.errorHandling.suppressDisplay = true;
            }
            throw error;
        }
    });
}
exports.getGitHubJsonResponse = getGitHubJsonResponse;
/**
 * @param label Property of JSON that will be used as the QuickPicks label
 * @param description Optional property of JSON that will be used as QuickPicks description
 * @param data Optional property of JSON that will be used as QuickPicks data saved as a NameValue pair
 */
function createQuickPickFromJsons(jsons, label) {
    const quickPicks = [];
    if (!util_1.isArray(jsons)) {
        jsons = [jsons];
    }
    for (const json of jsons) {
        if (!json[label]) {
            // skip this JSON if it doesn't have this label
            continue;
        }
        quickPicks.push({
            label: json[label],
            data: json
        });
    }
    return quickPicks;
}
exports.createQuickPickFromJsons = createQuickPickFromJsons;
function parseLinkHeaderToGitHubLinkObject(linkHeader) {
    const linkUrls = linkHeader.split(', ');
    const linkMap = {};
    // link header response is "<https://api.github.com/organizations/6154722/repos?page=2>; rel="prev", <https://api.github.com/organizations/6154722/repos?page=4>; rel="next""
    const relative = 'rel=';
    for (const url of linkUrls) {
        linkMap[url.substring(url.indexOf(relative) + relative.length + 1, url.length - 1)] = url.substring(url.indexOf('<') + 1, url.indexOf('>'));
    }
    return linkMap;
}
function getGitHubQuickPicksWithLoadMore(context, cache, originalUrl, labelName, timeoutSeconds = 10) {
    return __awaiter(this, void 0, void 0, function* () {
        const timeoutMs = timeoutSeconds * 1000;
        const startTime = Date.now();
        let gitHubQuickPicks = [];
        let url = originalUrl;
        do {
            const [result, nextLink] = yield getGitHubJsonResponse(context, url);
            gitHubQuickPicks = gitHubQuickPicks.concat(result);
            // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
            url = nextLink;
        } while (url && startTime + timeoutMs > Date.now());
        cache.picks = cache.picks.concat(createQuickPickFromJsons(gitHubQuickPicks, labelName));
        cache.picks.sort((a, b) => a.label.localeCompare(b.label));
        if (url) {
            return [{
                    label: '$(sync) Load More',
                    suppressPersistence: true,
                    data: url
                }].concat(cache.picks);
        }
        else {
            return cache.picks;
        }
    });
}
exports.getGitHubQuickPicksWithLoadMore = getGitHubQuickPicksWithLoadMore;
function createGitHubClient(context) {
    return __awaiter(this, void 0, void 0, function* () {
        const site = nonNull_1.nonNullProp(context, 'site');
        const client = yield site.createClient(context);
        const oAuth2Token = (yield client.listSourceControls())[0].token;
        if (!oAuth2Token) {
            yield showGitHubAuthPrompt(context, site);
            context.errorHandling.suppressDisplay = true;
            const noToken = localize_1.localize('noToken', 'No oAuth2 Token.');
            throw new Error(noToken);
        }
        return vscode_azureextensionui_1.createGenericClient(context, new ms_rest_js_1.TokenCredentials(oAuth2Token));
    });
}
exports.createGitHubClient = createGitHubClient;
//# sourceMappingURL=connectToGitHub.js.map