"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.startRemoteDebug = exports.RemoteDebugLanguage = void 0;
const portfinder = require("portfinder");
const vscode = require("vscode");
const vscode_azureextensionui_1 = require("vscode-azureextensionui");
const localize_1 = require("../localize");
const TunnelProxy_1 = require("../TunnelProxy");
const remoteDebugCommon_1 = require("./remoteDebugCommon");
const remoteDebugLink = 'https://aka.ms/appsvc-remotedebug';
let isRemoteDebugging = false;
var RemoteDebugLanguage;
(function (RemoteDebugLanguage) {
    RemoteDebugLanguage[RemoteDebugLanguage["Node"] = 0] = "Node";
    RemoteDebugLanguage[RemoteDebugLanguage["Python"] = 1] = "Python";
})(RemoteDebugLanguage = exports.RemoteDebugLanguage || (exports.RemoteDebugLanguage = {}));
function startRemoteDebug(context, site, siteConfig, language) {
    return __awaiter(this, void 0, void 0, function* () {
        if (isRemoteDebugging) {
            throw new Error(localize_1.localize('remoteDebugAlreadyStarted', 'Azure Remote Debugging is currently starting or already started.'));
        }
        isRemoteDebugging = true;
        try {
            yield startRemoteDebugInternal(context, site, siteConfig, language);
        }
        catch (error) {
            isRemoteDebugging = false;
            throw error;
        }
    });
}
exports.startRemoteDebug = startRemoteDebug;
function startRemoteDebugInternal(context, site, siteConfig, language) {
    return __awaiter(this, void 0, void 0, function* () {
        yield vscode.window.withProgress({ location: vscode.ProgressLocation.Notification, cancellable: true }, (progress, token) => __awaiter(this, void 0, void 0, function* () {
            const localHostPortNumber = yield portfinder.getPortPromise();
            const debugConfig = yield getDebugConfiguration(language, localHostPortNumber);
            const confirmEnableMessage = localize_1.localize('remoteDebugEnablePrompt', 'The configuration will be updated to enable remote debugging. Would you like to continue? This will restart the app.');
            yield remoteDebugCommon_1.setRemoteDebug(context, true, confirmEnableMessage, undefined, site, siteConfig, progress, token, remoteDebugLink);
            remoteDebugCommon_1.reportMessage(localize_1.localize('remoteDebugStartingTunnel', 'Starting tunnel proxy...'), progress, token);
            const client = yield site.createClient(context);
            const publishCredential = yield client.getWebAppPublishCredential();
            const tunnelProxy = new TunnelProxy_1.TunnelProxy(localHostPortNumber, site, publishCredential);
            yield vscode_azureextensionui_1.callWithTelemetryAndErrorHandling('appService.remoteDebugStartProxy', (startContext) => __awaiter(this, void 0, void 0, function* () {
                startContext.errorHandling.suppressDisplay = true;
                startContext.errorHandling.rethrow = true;
                yield tunnelProxy.startProxy(context, token);
            }));
            remoteDebugCommon_1.reportMessage(localize_1.localize('remoteDebugAttaching', 'Attaching debugger...'), progress, token);
            yield vscode_azureextensionui_1.callWithTelemetryAndErrorHandling('appService.remoteDebugAttach', (attachContext) => __awaiter(this, void 0, void 0, function* () {
                attachContext.errorHandling.suppressDisplay = true;
                attachContext.errorHandling.rethrow = true;
                yield vscode.debug.startDebugging(undefined, debugConfig);
            }));
            remoteDebugCommon_1.reportMessage(localize_1.localize('remoteDebugAttached', 'Attached!'), progress, token);
            const terminateDebugListener = vscode.debug.onDidTerminateDebugSession((event) => __awaiter(this, void 0, void 0, function* () {
                if (event.name === debugConfig.name) {
                    isRemoteDebugging = false;
                    if (tunnelProxy !== undefined) {
                        tunnelProxy.dispose();
                    }
                    terminateDebugListener.dispose();
                    const confirmDisableMessage = localize_1.localize('remoteDebugDisablePrompt', 'Remaining in debugging mode may cause performance issues. Would you like to disable debugging? This will restart the app.');
                    yield vscode.window.withProgress({ location: vscode.ProgressLocation.Notification, cancellable: true }, (innerProgress, innerToken) => __awaiter(this, void 0, void 0, function* () {
                        yield remoteDebugCommon_1.setRemoteDebug(context, false, confirmDisableMessage, undefined, site, siteConfig, innerProgress, innerToken, remoteDebugLink);
                    }));
                }
            }));
        }));
    });
}
function getDebugConfiguration(language, portNumber) {
    return __awaiter(this, void 0, void 0, function* () {
        const sessionId = Date.now().toString();
        const host = 'localhost';
        switch (language) {
            case RemoteDebugLanguage.Node:
                return yield getNodeDebugConfiguration(sessionId, portNumber, host);
            case RemoteDebugLanguage.Python:
                return yield getPythonDebugConfiguration(sessionId, portNumber, host);
            default:
                throw new Error(localize_1.localize('remoteDebugLanguageNotSupported', 'The language "{0}" is not supported for remote debugging.', language));
        }
    });
}
function getDebugPath() {
    return __awaiter(this, void 0, void 0, function* () {
        // Try to map workspace folder source files to the remote instance
        if (vscode.workspace.workspaceFolders) {
            if (vscode.workspace.workspaceFolders.length === 1) {
                return vscode.workspace.workspaceFolders[0].uri.fsPath;
            }
            else {
                // In this case we don't know which folder to use. Show a warning and proceed.
                // In the future we should allow users to choose a workspace folder to map sources from.
                const root = yield vscode.window.showWorkspaceFolderPick();
                if (root)
                    return root.uri.fsPath;
                else
                    throw new Error(localize_1.localize('remoteDebugNoFolders', 'Please select a workspace folder before attaching a debugger.'));
            }
        }
        else {
            // vscode will throw an error if you try to start debugging without any workspace folder open
            throw new Error(localize_1.localize('remoteDebugNoFolders', 'Please open a workspace folder before attaching a debugger.'));
        }
    });
}
function getNodeDebugConfiguration(sessionId, portNumber, host) {
    return __awaiter(this, void 0, void 0, function* () {
        const config = {
            name: sessionId,
            type: 'node',
            protocol: 'inspector',
            remoteRoot: '/home/site/wwwroot',
            request: 'attach',
            address: host,
            port: portNumber,
        };
        config.localRoot = yield getDebugPath();
        return config;
    });
}
function getPythonDebugConfiguration(sessionId, portNumber, host) {
    return __awaiter(this, void 0, void 0, function* () {
        const localRoot = yield getDebugPath();
        const config = {
            name: sessionId,
            type: 'python',
            request: 'attach',
            connect: {
                host: host,
                port: portNumber,
            },
            pathMappings: [
                {
                    localRoot: localRoot,
                    remoteRoot: '.',
                },
            ],
        };
        return config;
    });
}
//# sourceMappingURL=startRemoteDebug.js.map