"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.putFile = exports.listFiles = exports.getFile = void 0;
const ms_rest_js_1 = require("@azure/ms-rest-js");
const path = require("path");
const vscode_azureextensionui_1 = require("vscode-azureextensionui");
const createKuduClient_1 = require("./createKuduClient");
function getFile(context, site, filePath) {
    var _a;
    return __awaiter(this, void 0, void 0, function* () {
        let response;
        try {
            response = yield getFsResponse(context, site, filePath);
        }
        catch (error) {
            if (error instanceof ms_rest_js_1.RestError && error.code === 'PARSE_ERROR' && ((_a = error.response) === null || _a === void 0 ? void 0 : _a.status) === 200) {
                // Some files incorrectly list the content-type as json and fail to parse, but we always just want the text itself
                response = error.response;
            }
            else {
                throw error;
            }
        }
        return { data: response.bodyAsText, etag: response.headers.get('etag') };
    });
}
exports.getFile = getFile;
function listFiles(context, site, filePath) {
    return __awaiter(this, void 0, void 0, function* () {
        const response = yield getFsResponse(context, site, filePath);
        // eslint-disable-next-line @typescript-eslint/no-unsafe-return
        return Array.isArray(response.parsedBody) ? response.parsedBody : [];
    });
}
exports.listFiles = listFiles;
/**
 * Overwrites or creates a file. The etag passed in may be `undefined` if the file is being created
 * Returns the latest etag of the updated file
 */
function putFile(context, site, data, filePath, etag) {
    return __awaiter(this, void 0, void 0, function* () {
        const options = etag ? { customHeaders: { ['If-Match']: etag } } : {};
        const kuduClient = yield createKuduClient_1.createKuduClient(context, site);
        const result = (yield kuduClient.vfs.putItem(data, filePath, options))._response;
        return result.headers.get('etag');
    });
}
exports.putFile = putFile;
/**
 * Kudu APIs don't work for Linux consumption function apps and ARM APIs don't seem to work for web apps. We'll just have to use both
 */
function getFsResponse(context, site, filePath) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            if (site.isFunctionApp) {
                const linuxHome = '/home';
                if (site.isLinux && !filePath.startsWith(linuxHome)) {
                    filePath = path.posix.join(linuxHome, filePath);
                }
                const client = yield vscode_azureextensionui_1.createGenericClient(context, site.subscription);
                return yield client.sendRequest({
                    method: 'GET',
                    url: `${site.id}/hostruntime/admin/vfs/${filePath}?api-version=2018-11-01`
                });
            }
            else {
                const kuduClient = yield createKuduClient_1.createKuduClient(context, site);
                return (yield kuduClient.vfs.getItem(filePath))._response;
            }
        }
        catch (error) {
            context.telemetry.maskEntireErrorMessage = true; // since the error could have the contents of the user's file
            throw error;
        }
    });
}
//# sourceMappingURL=siteFiles.js.map