"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.stopStreamingLogs = exports.startStreamingLogs = void 0;
const abort_controller_1 = require("@azure/abort-controller");
const ms_rest_js_1 = require("@azure/ms-rest-js");
const timers_1 = require("timers");
const vscode = require("vscode");
const vscode_azureextensionui_1 = require("vscode-azureextensionui");
const extensionVariables_1 = require("./extensionVariables");
const localize_1 = require("./localize");
const pingFunctionApp_1 = require("./pingFunctionApp");
const nonNull_1 = require("./utils/nonNull");
const logStreams = new Map();
function getLogStreamId(site, logsPath) {
    return `${site.id}${logsPath}`;
}
function startStreamingLogs(context, site, verifyLoggingEnabled, logStreamLabel, logsPath = '') {
    return __awaiter(this, void 0, void 0, function* () {
        const logStreamId = getLogStreamId(site, logsPath);
        const logStream = logStreams.get(logStreamId);
        if (logStream && logStream.isConnected) {
            logStream.outputChannel.show();
            void context.ui.showWarningMessage(localize_1.localize('logStreamAlreadyActive', 'The log-streaming service for "{0}" is already active.', logStreamLabel));
            return logStream;
        }
        else {
            yield verifyLoggingEnabled();
            const outputChannel = logStream ? logStream.outputChannel : vscode.window.createOutputChannel(localize_1.localize('logStreamLabel', '{0} - Log Stream', logStreamLabel));
            extensionVariables_1.ext.context.subscriptions.push(outputChannel);
            outputChannel.show();
            outputChannel.appendLine(localize_1.localize('connectingToLogStream', 'Connecting to log stream...'));
            const client = yield site.createClient(context);
            const creds = yield client.getWebAppPublishCredential();
            return yield new Promise((onLogStreamCreated) => {
                // Intentionally setting up a separate telemetry event and not awaiting the result here since log stream is a long-running action
                void vscode_azureextensionui_1.callWithTelemetryAndErrorHandling('appService.streamingLogs', (streamContext) => __awaiter(this, void 0, void 0, function* () {
                    streamContext.errorHandling.suppressDisplay = true;
                    let timerId;
                    if (site.isFunctionApp) {
                        // For Function Apps, we have to ping "/admin/host/status" every minute for logging to work
                        // https://github.com/Microsoft/vscode-azurefunctions/issues/227
                        yield pingFunctionApp_1.pingFunctionApp(streamContext, site);
                        // eslint-disable-next-line @typescript-eslint/no-misused-promises
                        timerId = timers_1.setInterval(() => __awaiter(this, void 0, void 0, function* () { return yield pingFunctionApp_1.pingFunctionApp(streamContext, site); }), 60 * 1000);
                    }
                    const genericClient = yield vscode_azureextensionui_1.createGenericClient(streamContext, new ms_rest_js_1.BasicAuthenticationCredentials(creds.publishingUserName, nonNull_1.nonNullProp(creds, 'publishingPassword')));
                    const abortController = new abort_controller_1.AbortController();
                    const logsResponse = yield genericClient.sendRequest({
                        method: 'GET',
                        url: `${site.kuduUrl}/api/logstream/${logsPath}`,
                        streamResponseBody: true,
                        abortSignal: abortController.signal
                    });
                    yield new Promise((onLogStreamEnded, reject) => {
                        var _a;
                        const newLogStream = {
                            dispose: () => {
                                var _a;
                                (_a = logsResponse.readableStreamBody) === null || _a === void 0 ? void 0 : _a.removeAllListeners();
                                abortController.abort();
                                outputChannel.show();
                                if (timerId) {
                                    clearInterval(timerId);
                                }
                                outputChannel.appendLine(localize_1.localize('logStreamDisconnected', 'Disconnected from log-streaming service.'));
                                newLogStream.isConnected = false;
                                void onLogStreamEnded();
                            },
                            isConnected: true,
                            outputChannel: outputChannel
                        };
                        (_a = logsResponse.readableStreamBody) === null || _a === void 0 ? void 0 : _a.on('data', (chunk) => {
                            outputChannel.append(chunk.toString());
                        }).on('error', (err) => {
                            if (timerId) {
                                clearInterval(timerId);
                            }
                            newLogStream.isConnected = false;
                            outputChannel.show();
                            outputChannel.appendLine(localize_1.localize('logStreamError', 'Error connecting to log-streaming service:'));
                            outputChannel.appendLine(vscode_azureextensionui_1.parseError(err).message);
                            reject(err);
                        }).on('complete', () => {
                            newLogStream.dispose();
                        });
                        logStreams.set(logStreamId, newLogStream);
                        onLogStreamCreated(newLogStream);
                    });
                }));
            });
        }
    });
}
exports.startStreamingLogs = startStreamingLogs;
function stopStreamingLogs(site, logsPath = '') {
    return __awaiter(this, void 0, void 0, function* () {
        const logStreamId = getLogStreamId(site, logsPath);
        const logStream = logStreams.get(logStreamId);
        if (logStream && logStream.isConnected) {
            logStream.dispose();
        }
        else {
            yield vscode.window.showWarningMessage(localize_1.localize('alreadyDisconnected', 'The log-streaming service is already disconnected.'));
        }
    });
}
exports.stopStreamingLogs = stopStreamingLogs;
//# sourceMappingURL=startStreamingLogs.js.map