"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.AppSettingTreeItem = void 0;
const vscode_1 = require("vscode");
const vscode_azureextensionui_1 = require("vscode-azureextensionui");
const extensionVariables_1 = require("../extensionVariables");
const localize_1 = require("../localize");
const AppSettingsTreeItem_1 = require("./AppSettingsTreeItem");
/**
 * NOTE: This leverages a command with id `ext.prefix + '.toggleAppSettingVisibility'` that should be registered by each extension
 */
class AppSettingTreeItem extends vscode_azureextensionui_1.AzExtTreeItem {
    constructor(parent, key, value) {
        super(parent);
        this._key = key;
        this._value = value;
        this._hideValue = true;
        this.valuesToMask.push(key, value);
    }
    get contextValue() {
        return this.parent.supportsSlots ? AppSettingTreeItem.contextValue : AppSettingTreeItem.contextValueNoSlots;
    }
    static createAppSettingTreeItem(context, parent, key, value) {
        return __awaiter(this, void 0, void 0, function* () {
            const ti = new AppSettingTreeItem(parent, key, value);
            // check if it's a slot setting
            yield ti.refreshImpl(context);
            return ti;
        });
    }
    get id() {
        return this._key;
    }
    get label() {
        return this._hideValue ? `${this._key}=Hidden value. Click to view.` : `${this._key}=${this._value}`;
    }
    get iconPath() {
        return new vscode_1.ThemeIcon('symbol-constant');
    }
    get commandId() {
        return extensionVariables_1.ext.prefix + '.toggleAppSettingVisibility';
    }
    edit(context) {
        return __awaiter(this, void 0, void 0, function* () {
            const newValue = yield context.ui.showInputBox({
                prompt: `Enter setting value for "${this._key}"`,
                stepName: 'appSettingValue',
                value: this._value
            });
            yield this.parent.editSettingItem(this._key, this._key, newValue, context);
            this._value = newValue;
            yield this.refresh(context);
        });
    }
    rename(context) {
        return __awaiter(this, void 0, void 0, function* () {
            const settings = yield this.parent.ensureSettings(context);
            const client = yield this.parent.clientProvider.createClient(context);
            const oldKey = this._key;
            const newKey = yield context.ui.showInputBox({
                prompt: `Enter a new name for "${oldKey}"`,
                stepName: 'appSettingName',
                value: this._key,
                validateInput: (v) => AppSettingsTreeItem_1.validateAppSettingKey(settings, client, v, oldKey)
            });
            yield this.parent.editSettingItem(oldKey, newKey, this._value, context);
            this._key = newKey;
            yield this.refresh(context);
        });
    }
    deleteTreeItemImpl(context) {
        return __awaiter(this, void 0, void 0, function* () {
            yield context.ui.showWarningMessage(`Are you sure you want to delete setting "${this._key}"?`, { modal: true, stepName: 'confirmDelete' }, vscode_azureextensionui_1.DialogResponses.deleteResponse);
            yield this.parent.deleteSettingItem(this._key, context);
        });
    }
    toggleValueVisibility(context) {
        return __awaiter(this, void 0, void 0, function* () {
            this._hideValue = !this._hideValue;
            yield this.refresh(context);
        });
    }
    toggleSlotSetting(context) {
        return __awaiter(this, void 0, void 0, function* () {
            const client = yield this.parent.clientProvider.createClient(context);
            if (client.updateSlotConfigurationNames && client.listSlotConfigurationNames) {
                const slotSettings = yield client.listSlotConfigurationNames();
                if (!slotSettings.appSettingNames) {
                    slotSettings.appSettingNames = [];
                }
                const slotSettingIndex = slotSettings.appSettingNames.findIndex((value) => { return value === this._key; });
                if (slotSettingIndex >= 0) {
                    slotSettings.appSettingNames.splice(slotSettingIndex, 1);
                }
                else {
                    slotSettings.appSettingNames.push(this._key);
                }
                yield client.updateSlotConfigurationNames(slotSettings);
                yield this.refresh(context);
            }
            else {
                throw Error(localize_1.localize('toggleSlotSettingsNotSupported', 'Toggling slot settings is not supported.'));
            }
        });
    }
    refreshImpl(context) {
        return __awaiter(this, void 0, void 0, function* () {
            const client = yield this.parent.clientProvider.createClient(context);
            if (client.listSlotConfigurationNames) {
                const slotSettings = yield client.listSlotConfigurationNames();
                if (slotSettings.appSettingNames && slotSettings.appSettingNames.find((value) => { return value === this._key; })) {
                    this.description = localize_1.localize('slotSetting', 'Slot Setting');
                }
                else {
                    this.description = undefined;
                }
            }
        });
    }
}
exports.AppSettingTreeItem = AppSettingTreeItem;
AppSettingTreeItem.contextValue = 'applicationSettingItem';
AppSettingTreeItem.contextValueNoSlots = 'applicationSettingItemNoSlots';
//# sourceMappingURL=AppSettingTreeItem.js.map