"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.AppSettingsTreeItem = exports.validateAppSettingKey = void 0;
const vscode_1 = require("vscode");
const vscode_azureextensionui_1 = require("vscode-azureextensionui");
const AppSettingTreeItem_1 = require("./AppSettingTreeItem");
function validateAppSettingKey(settings, client, newKey, oldKey) {
    if (client.isLinux && /[^\w\.]+/.test(newKey)) {
        return 'App setting names can only contain letters, numbers (0-9), periods ("."), and underscores ("_")';
    }
    newKey = newKey.trim();
    if (newKey.length === 0) {
        return 'App setting names must have at least one non-whitespace character.';
    }
    oldKey = oldKey ? oldKey.trim().toLowerCase() : oldKey;
    if (settings.properties && newKey.toLowerCase() !== oldKey) {
        for (const key of Object.keys(settings.properties)) {
            if (key.toLowerCase() === newKey.toLowerCase()) {
                return `App setting name "${newKey}" already exists.`;
            }
        }
    }
    return undefined;
}
exports.validateAppSettingKey = validateAppSettingKey;
class AppSettingsTreeItem extends vscode_azureextensionui_1.AzExtParentTreeItem {
    constructor(parent, clientProvider, supportsSlots = true, settingsToHide) {
        super(parent);
        this.label = 'Application Settings';
        this.childTypeLabel = 'App Setting';
        this.contextValue = AppSettingsTreeItem.contextValue;
        this.suppressMaskLabel = true;
        this.clientProvider = clientProvider;
        this.supportsSlots = supportsSlots;
        this._settingsToHide = settingsToHide;
    }
    get id() {
        return 'configuration';
    }
    get iconPath() {
        return new vscode_1.ThemeIcon('settings');
    }
    hasMoreChildrenImpl() {
        return false;
    }
    loadMoreChildrenImpl(_clearCache, context) {
        return __awaiter(this, void 0, void 0, function* () {
            const client = yield this.clientProvider.createClient(context);
            this._settings = yield client.listApplicationSettings();
            const treeItems = [];
            const properties = this._settings.properties || {};
            yield Promise.all(Object.keys(properties).map((key) => __awaiter(this, void 0, void 0, function* () {
                var _a;
                const appSettingTreeItem = yield AppSettingTreeItem_1.AppSettingTreeItem.createAppSettingTreeItem(context, this, key, properties[key]);
                if (!((_a = this._settingsToHide) === null || _a === void 0 ? void 0 : _a.includes(key))) {
                    treeItems.push(appSettingTreeItem);
                }
            })));
            return treeItems;
        });
    }
    editSettingItem(oldKey, newKey, value, context) {
        return __awaiter(this, void 0, void 0, function* () {
            // make a deep copy so settings are not cached if there's a failure
            // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
            const settings = JSON.parse(JSON.stringify(yield this.ensureSettings(context)));
            if (settings.properties) {
                if (oldKey !== newKey) {
                    delete settings.properties[oldKey];
                }
                settings.properties[newKey] = value;
            }
            const client = yield this.clientProvider.createClient(context);
            this._settings = yield client.updateApplicationSettings(settings);
        });
    }
    deleteSettingItem(key, context) {
        return __awaiter(this, void 0, void 0, function* () {
            // make a deep copy so settings are not cached if there's a failure
            // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
            const settings = JSON.parse(JSON.stringify(yield this.ensureSettings(context)));
            if (settings.properties) {
                delete settings.properties[key];
            }
            const client = yield this.clientProvider.createClient(context);
            this._settings = yield client.updateApplicationSettings(settings);
        });
    }
    createChildImpl(context) {
        return __awaiter(this, void 0, void 0, function* () {
            const client = yield this.clientProvider.createClient(context);
            // make a deep copy so settings are not cached if there's a failure
            // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
            const settings = JSON.parse(JSON.stringify(yield this.ensureSettings(context)));
            const newKey = yield context.ui.showInputBox({
                prompt: 'Enter new app setting name',
                stepName: 'appSettingName',
                validateInput: (v) => validateAppSettingKey(settings, client, v)
            });
            const newValue = yield context.ui.showInputBox({
                prompt: `Enter value for "${newKey}"`,
                stepName: 'appSettingValue'
            });
            if (!settings.properties) {
                settings.properties = {};
            }
            context.showCreatingTreeItem(newKey);
            settings.properties[newKey] = newValue;
            this._settings = yield client.updateApplicationSettings(settings);
            return yield AppSettingTreeItem_1.AppSettingTreeItem.createAppSettingTreeItem(context, this, newKey, newValue);
        });
    }
    ensureSettings(context) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this._settings) {
                yield this.getCachedChildren(context);
            }
            return this._settings;
        });
    }
}
exports.AppSettingsTreeItem = AppSettingsTreeItem;
AppSettingsTreeItem.contextValue = 'applicationSettings';
//# sourceMappingURL=AppSettingsTreeItem.js.map