"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.DeploymentTreeItem = void 0;
const os = require("os");
const vscode_1 = require("vscode");
const vscode_azureextensionui_1 = require("vscode-azureextensionui");
const createKuduClient_1 = require("../createKuduClient");
const waitForDeploymentToComplete_1 = require("../deploy/waitForDeploymentToComplete");
const extensionVariables_1 = require("../extensionVariables");
const localize_1 = require("../localize");
const kuduUtils_1 = require("../utils/kuduUtils");
const nonNull_1 = require("../utils/nonNull");
const openUrl_1 = require("../utils/openUrl");
// Kudu DeployStatus: https://github.com/projectkudu/kudu/blob/a13592e6654585d5c2ee5c6a05fa39fa812ebb84/Kudu.Contracts/Deployment/DeployStatus.cs
var DeployStatus;
(function (DeployStatus) {
    DeployStatus[DeployStatus["Building"] = 0] = "Building";
    DeployStatus[DeployStatus["Deploying"] = 1] = "Deploying";
    DeployStatus[DeployStatus["Pending"] = 2] = "Pending";
    DeployStatus[DeployStatus["Failed"] = 3] = "Failed";
    DeployStatus[DeployStatus["Success"] = 4] = "Success";
})(DeployStatus || (DeployStatus = {}));
/**
 * NOTE: This leverages two commands prefixed with `ext.prefix` that should be registered by each extension: "showOutputChannel" and "viewDeploymentLogs"
 */
class DeploymentTreeItem extends vscode_azureextensionui_1.AzExtTreeItem {
    constructor(parent, deployResult, scmType) {
        super(parent);
        this.contextValue = `deployment/${scmType}`.toLocaleLowerCase();
        this._deployResult = deployResult;
        this.receivedTime = nonNull_1.nonNullProp(deployResult, 'receivedTime');
        const message = this.getDeploymentMessage(deployResult);
        this.label = `${this.id.substring(0, 7)} - ${message}`;
    }
    get iconPath() {
        return new vscode_1.ThemeIcon('git-commit');
    }
    get id() {
        this._deployResult.id = nonNull_1.nonNullProp(this._deployResult, 'id');
        return this._deployResult.id;
    }
    get commandId() {
        return `${extensionVariables_1.ext.prefix}.viewDeploymentLogs`;
    }
    get description() {
        if (this._deployResult.active) {
            return localize_1.localize('active', 'Active');
        }
        switch (this._deployResult.status) {
            case DeployStatus.Building:
                return localize_1.localize('building', 'Building...');
            case DeployStatus.Deploying:
                return localize_1.localize('deploying', 'Deploying...');
            case DeployStatus.Pending:
                return localize_1.localize('pending', 'Pending...');
            case DeployStatus.Failed:
                return localize_1.localize('failed', 'Failed');
            case DeployStatus.Success:
            default:
                return;
        }
    }
    isAncestorOfImpl(contextValue) {
        return this.contextValue === contextValue;
    }
    redeployDeployment(context) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this._deployResult.isReadonly) {
                throw new Error(localize_1.localize('redeployNotSupported', 'Redeploy is not supported for non-git deployments.'));
            }
            const redeploying = localize_1.localize('redeploying', 'Redeploying commit "{0}" to "{1}". Check [output window](command:{2}) for status.', this.id, this.parent.site.fullName, extensionVariables_1.ext.prefix + '.showOutputChannel');
            const redeployed = localize_1.localize('redeployed', 'Commit "{0}" has been redeployed to "{1}".', this.id, this.parent.site.fullName);
            yield vscode_1.window.withProgress({ location: vscode_1.ProgressLocation.Notification, title: redeploying }, () => __awaiter(this, void 0, void 0, function* () {
                extensionVariables_1.ext.outputChannel.appendLog(localize_1.localize('reployingOutput', 'Redeploying commit "{0}" to "{1}"...', this.id, this.parent.site.fullName), { resourceName: this.parent.site.fullName });
                const kuduClient = yield createKuduClient_1.createKuduClient(context, this.parent.site);
                void kuduClient.deployment.deploy(this.id);
                // eslint-disable-next-line @typescript-eslint/no-misused-promises
                const refreshingInteveral = setInterval(() => __awaiter(this, void 0, void 0, function* () { yield this.refresh(context); }), 1000); /* the status of the label changes during deployment so poll for that*/
                try {
                    yield waitForDeploymentToComplete_1.waitForDeploymentToComplete(context, this.parent.site, this.id);
                    yield this.parent.refresh(context); /* refresh entire node because active statuses has changed */
                    void vscode_1.window.showInformationMessage(redeployed);
                    extensionVariables_1.ext.outputChannel.appendLog(redeployed);
                }
                finally {
                    clearInterval(refreshingInteveral);
                }
            }));
        });
    }
    getDeploymentLogs(context) {
        return __awaiter(this, void 0, void 0, function* () {
            const kuduClient = yield createKuduClient_1.createKuduClient(context, this.parent.site);
            let logEntries = [];
            yield kuduUtils_1.retryKuduCall(context, 'getLogEntry', () => __awaiter(this, void 0, void 0, function* () {
                yield kuduUtils_1.ignore404Error(context, () => __awaiter(this, void 0, void 0, function* () {
                    logEntries = yield kuduClient.deployment.getLogEntry(this.id);
                }));
            }));
            let data = '';
            for (const logEntry of logEntries) {
                data += this.formatLogEntry(logEntry);
                let detailedLogEntries = [];
                yield kuduUtils_1.retryKuduCall(context, 'getLogEntryDetails', () => __awaiter(this, void 0, void 0, function* () {
                    yield kuduUtils_1.ignore404Error(context, () => __awaiter(this, void 0, void 0, function* () {
                        if (logEntry.detailsUrl && logEntry.id) {
                            detailedLogEntries = yield kuduClient.deployment.getLogEntryDetails(this.id, logEntry.id);
                        }
                    }));
                }));
                for (const detailedEntry of detailedLogEntries) {
                    data += this.formatLogEntry(detailedEntry);
                }
            }
            return data;
        });
    }
    viewDeploymentLogs(context) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.runWithTemporaryDescription(context, localize_1.localize('retrievingLogs', 'Retrieving logs...'), () => __awaiter(this, void 0, void 0, function* () {
                const logData = yield this.getDeploymentLogs(context);
                yield vscode_azureextensionui_1.openReadOnlyContent(this, logData, '.log');
            }));
        });
    }
    viewCommitInGitHub(context) {
        return __awaiter(this, void 0, void 0, function* () {
            const client = yield this.parent.site.createClient(context);
            const sourceControl = yield client.getSourceControl();
            if (sourceControl.repoUrl) {
                const gitHubCommitUrl = `${sourceControl.repoUrl}/commit/${this._deployResult.id}`;
                yield openUrl_1.openUrl(gitHubCommitUrl);
                return;
            }
            else {
                throw new Error(localize_1.localize('noRepoUrl', 'There is no GitHub repo url associated with deployment "{0}".', this._deployResult.id));
            }
        });
    }
    refreshImpl(context) {
        return __awaiter(this, void 0, void 0, function* () {
            const kuduClient = yield createKuduClient_1.createKuduClient(context, this.parent.site);
            this._deployResult = yield kuduClient.deployment.getResult(this.id);
        });
    }
    formatLogEntry(logEntry) {
        if (logEntry.logTime && logEntry.message) {
            return `${logEntry.logTime.toISOString()} - ${logEntry.message}${os.EOL}`;
        }
        else {
            return '';
        }
    }
    getDeploymentMessage(deployResult) {
        let message = nonNull_1.nonNullProp(deployResult, 'message');
        try {
            const messageJSON = JSON.parse(message);
            if (messageJSON.message) {
                message = messageJSON.message;
            }
        }
        catch (_a) {
            // Ignore and assume message was not in JSON format
        }
        const firstLine = this.getFirstLine(message);
        /* truncate long messages and add "..." */
        return firstLine.length > 50 ? `${firstLine.substring(0, 50)}...` : firstLine;
    }
    getFirstLine(message) {
        const allLineBreaks = /\r?\n|\r/;
        const index = message.search(allLineBreaks);
        if (index >= 0) {
            message = message.substring(0, index);
        }
        return message;
    }
}
exports.DeploymentTreeItem = DeploymentTreeItem;
DeploymentTreeItem.contextValue = new RegExp('deployment\/.*');
//# sourceMappingURL=DeploymentTreeItem.js.map