"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.DeploymentsTreeItem = void 0;
const vscode_1 = require("vscode");
const vscode_azureextensionui_1 = require("vscode-azureextensionui");
const createKuduClient_1 = require("../createKuduClient");
const extensionVariables_1 = require("../extensionVariables");
const localize_1 = require("../localize");
const ScmType_1 = require("../ScmType");
const kuduUtils_1 = require("../utils/kuduUtils");
const DeploymentTreeItem_1 = require("./DeploymentTreeItem");
/**
 * NOTE: This leverages a command with id `ext.prefix + '.connectToGitHub'` that should be registered by each extension
 */
class DeploymentsTreeItem extends vscode_azureextensionui_1.AzExtParentTreeItem {
    constructor(parent, site, siteConfig, sourceControl) {
        super(parent);
        this.label = localize_1.localize('Deployments', 'Deployments');
        this.childTypeLabel = localize_1.localize('Deployment', 'Deployment');
        this.suppressMaskLabel = true;
        this.site = site;
        this._scmType = siteConfig.scmType;
        this._repoUrl = sourceControl.repoUrl;
    }
    get iconPath() {
        return new vscode_1.ThemeIcon('cloud-upload');
    }
    get description() {
        switch (this._scmType) {
            case ScmType_1.ScmType.LocalGit:
                return localize_1.localize('git', 'Git');
            case ScmType_1.ScmType.GitHub:
                // remove github from the repoUrl which leaves only the org/repo names
                return this._repoUrl ? this._repoUrl.substring('https://github.com/'.length) : localize_1.localize('gitHub', 'GitHub');
            case ScmType_1.ScmType.None:
            default:
                return '';
        }
    }
    get contextValue() {
        return this._scmType === ScmType_1.ScmType.None ? DeploymentsTreeItem.contextValueUnconnected : DeploymentsTreeItem.contextValueConnected;
    }
    hasMoreChildrenImpl() {
        return false;
    }
    loadMoreChildrenImpl(_clearCache, context) {
        return __awaiter(this, void 0, void 0, function* () {
            const client = yield this.site.createClient(context);
            const siteConfig = yield client.getSiteConfig();
            const kuduClient = yield createKuduClient_1.createKuduClient(context, this.site);
            const deployments = yield kuduUtils_1.retryKuduCall(context, 'getDeployResults', () => __awaiter(this, void 0, void 0, function* () {
                return kuduClient.deployment.getDeployResults();
            }));
            const children = yield this.createTreeItemsWithErrorHandling(deployments, 'invalidDeployment', dr => {
                return new DeploymentTreeItem_1.DeploymentTreeItem(this, dr, siteConfig.scmType);
            }, dr => {
                return dr.id ? dr.id.substring(0, 7) : undefined;
            });
            if (siteConfig.scmType === ScmType_1.ScmType.None) {
                // redeploy does not support Push deploys, so we still guide users to connect to a GitHub repo
                children.push(new vscode_azureextensionui_1.GenericTreeItem(this, {
                    commandId: extensionVariables_1.ext.prefix + '.connectToGitHub',
                    contextValue: 'ConnectToGithub',
                    label: 'Connect to a GitHub Repository...'
                }));
            }
            return children;
        });
    }
    compareChildrenImpl(ti1, ti2) {
        if (ti1 instanceof vscode_azureextensionui_1.GenericTreeItem) {
            return 1;
        }
        else if (ti2 instanceof vscode_azureextensionui_1.GenericTreeItem) {
            return -1;
        }
        // sorts in accordance of the most recent deployment
        return ti2.receivedTime.valueOf() - ti1.receivedTime.valueOf();
    }
    refreshImpl(context) {
        return __awaiter(this, void 0, void 0, function* () {
            const client = yield this.site.createClient(context);
            const siteConfig = yield client.getSiteConfig();
            const sourceControl = yield client.getSourceControl();
            this._scmType = siteConfig.scmType;
            this._repoUrl = sourceControl.repoUrl;
        });
    }
}
exports.DeploymentsTreeItem = DeploymentsTreeItem;
DeploymentsTreeItem.contextValueConnected = 'deploymentsConnected';
DeploymentsTreeItem.contextValueUnconnected = 'deploymentsUnconnected';
//# sourceMappingURL=DeploymentsTreeItem.js.map